<?php
/**
 * YANA library
 *
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package  yana
 * @license  http://www.gnu.org/licenses/gpl.txt
 */

/**
 * handle php files
 *
 * This is a wrapper class for executeable PHP files.
 *
 * @access      public
 * @package     yana
 * @subpackage  file_system
 */
class Executeable extends FileSystemResource
{

    /**
    * require / include a php file
    *
    * Tries to interpret the given file as php code using "require_once()"
    *
    * This implements dynamic loading of php source code.
    *
    * @uses $executeable->read()
    *
    * @access  public
    * @return  bool
    */
    function read()
    {
        if ($this->exists()) {
            require_once($this->getPath());
            return true;
        } else {
            return false;
        }
    }

    /**
    * alias of toString
    *
    * @ignore
    */
    function get()
    {
        return $this->toString();
    }

    /**
    * get the filename
    *
    * @uses $executeable->toString()
    *
    * @access  public
    * @return  string
    *
    * @ignore
    */
    function toString()
    {
        if (!$this->exists()) {
            return "PHP-File ".$this->getPath()." does not exist\n";
        } elseif ($this->isEmpty()) {
            return "PHP-File ".$this->getPath()." is wether empty or not loaded\n";
        } else {
            return "PHP-File \"".$this->getPath()."\"\n";
        }
    }

}
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
