<?php
/**
 * YANA library
 *
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package  yana
 * @license  http://www.gnu.org/licenses/gpl.txt
 */

/**
 * Message
 *
 * This is a super class for all common messages.
 * It may be used to represent information, that
 * is not an error. E.g. reporting, that a certain
 * operation has been finished successfully.
 *
 * @access      public
 * @package     yana
 * @subpackage  error_reporting
 */
class Message extends Report
{
    /**
     * constructor
     *
     * @name    Message::__construct()
     * @param   array|string  $message
     * @param   mixed         $data    (optional)
     */
    function __construct($message, $data = null)
    {
        /* Implicitely allow int as input, so constants may be used that are resolved later at runtime */
        if (is_string($message) || is_int($message)) {
            /* settype to STRING */
            $message = (string) $message;
            $this->message = $message;
        } elseif (is_array($message)) {
            if (count($message) == 2 && isset($message[0]) && isset($message[1])) {
                $this->message = array('H' => $message[0],'P' => $message[1]);
            } else {
                $message = Hashtable::changeCase($message,CASE_UPPER);
                if (count($message) == 2 && isset($message['H']) && isset($message['P'])) {
                    $this->message = $message;
                } else {
                    trigger_error("Wrong argument type for argument 1 in ".__CLASS__."->".__FUNCTION__."().\n\t\tExpected a string or an array with the 2 elements, the first being the message's name and the second being the message description.\n\t\tInstead found '".gettype($message)."(".print_r($message,1).")'");
                }
            }
        } else {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, "1 in ".__CLASS__."->".__FUNCTION__."()", 'String or array', gettype($message)), E_USER_WARNING);
        }
        $this->data = $data;
    }

    /**
     * getMessage
     *
     * returns the message string of this event
     *
     * <ol>
     *     <li>    checks whether message is fulltext or id and resolves id,            </li>
     *     <li>    checks whether data is array or empty or other and replaces token    </li>
     * </ol>
     *
     * @access  public
     * @name    Message::getMessage()
     * @return  mixed
     */
    function getMessage()
    {
        global $YANA;
        $message = "";

        /* 1) */
        if (isset($YANA) && is_string($this->message) && preg_match('/^[\w_\d]+$/', $this->message)) {
            $message = $YANA->getVar('TEXT.'.strtoupper($this->message));
        } else {
            $message = $this->message;
        }
        if (empty($message)) {
            $message = $this->message;
        /* 2) */
        } elseif (is_array($message) && !empty($this->data) && is_array($this->data)) {
            if (!empty($message['H'])) {
                $message['H'] = SmartUtility::fillTemplate($message['H'], $this->data);
            }
            if (!empty($message['P'])) {
                $message['P'] = SmartUtility::fillTemplate($message['P'], $this->data);
            }
        }
        return $message;
    }
}
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
