<?php
/**
 * Fall back functions
 *
 * This file is to collect fall-backs for PHP 4, in order to enhance backwards compatibility.
 *
 * These will apply automatically.
 * If the original function does not exist in the installed version of PHP,
 * the fall-back function listed here will fill the gap so your code will not be broken.
 *
 * Of course these implementations won't reach the same performance as the original functions -
 * but they will work.
 *
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package     yana
 * @subpackage  utilities
 * @license     http://www.gnu.org/licenses/gpl.txt
 * @ignore
 */

/**#@+
 * CONSTANTS
 *
 * @ignore
 */

/* the following applies to (among others)
 *
 * ARRAY_CHANGE_KEY_CASE()
 *
 * introduced in PHP 4.2.0
 */
if (!defined('CASE_LOWER')) {
    define('CASE_LOWER', 0);
}
if (!defined('CASE_UPPER')) {
    define('CASE_UPPER', 1);
}
/* introduced in PHP 4.2.0 */
if (!defined('PHP_SAPI')) {
    define('PHP_SAPI', php_sapi_name());
}

/* the following applies to
 *
 * $_FILES['foo']['error']
 *
 * introduced in PHP 4.3.0
 */
if (!defined('UPLOAD_ERR_OK')) {
    define('UPLOAD_ERR_OK', 0);
}
if (!defined('UPLOAD_ERR_INI_SIZE')) {
    define('UPLOAD_ERR_INI_SIZE', 1);
}
if (!defined('UPLOAD_ERR_FORM_SIZE')) {
    define('UPLOAD_ERR_FORM_SIZE', 2);
}
if (!defined('UPLOAD_ERR_PARTIAL')) {
    define('UPLOAD_ERR_PARTIAL', 3);
}
if (!defined('UPLOAD_ERR_NO_FILE')) {
    define('UPLOAD_ERR_NO_FILE', 4);
}
if (!defined('UPLOAD_ERR_NO_TMP_DIR')) {
    define('UPLOAD_ERR_NO_TMP_DIR', 5);
}
/* introduced in PHP 4.3.0 */
if (!defined("PATH_SEPARATOR")) {
    if (DIRECTORY_SEPARATOR === '\\') {
        define("PATH_SEPARATOR", ';'); /* Win32 */
    } else {
        define("PATH_SEPARATOR", ':'); /* other */
    }
}
if (!defined("PHP_SHLIB_SUFFIX")) {
    if (DIRECTORY_SEPARATOR === '\\') {
        define("PHP_SHLIB_SUFFIX", 'dll'); /* Win32 */
    } else {
        define("PHP_SHLIB_SUFFIX", 'so'); /* other */
    }
}
if (!defined("PHP_PREFIX")) {
    if (isset($_ENV["_"])) {
        define("PHP_PREFIX", substr($_ENV["_"], 0, strpos($_ENV["_"], "bin/")));
    }
}

/* The following applies to fnmatch()
 * introduced in PHP 4.3.0
 */
/* no wildcard ever matches a "/" */
if (!defined("FNM_PATHNAME")) {
    define("FNM_PATHNAME", 1<<0);
}
/* backslash can't escape meta chars */
if (!defined("FNM_NOESCAPE")) {
    define("FNM_NOESCAPE", 1<<1);
}
/* leading dot must be given explicit */
if (!defined("FNM_PERIOD")) {
    define("FNM_PERIOD", 1<<2);
}
if (!defined("FNM_LEADING_DIR")) {
    define("FNM_LEADING_DIR", 1<<3);
}
/* match case-insensitive */
if (!defined("FNM_CASEFOLD")) {
    define("FNM_CASEFOLD", 0x50);
}
if (!defined("FNM_EXTMATCH")) {
    define("FNM_EXTMATCH", 1<<5);
}
/* The following applies to glob()
 * introduced in PHP 4.3.0
 */
if (!defined("GLOB_MARK")) {
    define("GLOB_MARK", 1<<0);
}
if (!defined("GLOB_NOSORT")) {
    define("GLOB_NOSORT", 1<<1);
}
if (!defined("GLOB_NOCHECK")) {
    define("GLOB_NOCHECK", 1<<2);
}
if (!defined("GLOB_NOESCAPE")) {
    define("GLOB_NOESCAPE", 1<<3);
}
if (!defined("GLOB_BRACE")) {
    define("GLOB_BRACE", 1<<4);
}
if (!defined("GLOB_ONLYDIR")) {
    define("GLOB_ONLYDIR", 1<<5);
}
if (!defined("GLOB_NOCASE")) {
    define("GLOB_NOCASE", 1<<6);
}
if (!defined("GLOB_DOTS")) {
    define("GLOB_DOTS", 1<<7);
}

/* introduced in PHP 4.3.10, 5.0.2 */
if (!defined("PHP_EOL")) {
    if (DIRECTORY_SEPARATOR === "\\") { /* Win32 */
        define("PHP_EOL", "\015\012");
    } elseif (strncmp(PHP_OS, "D", 1)) { /* Mac OS */
        define("PHP_EOL", "\012");
    } else { /* Unix, Linux */
        define("PHP_EOL", "\015");
    }
}

/* introduced in PHP 4.3.10, 5.0.3 */
if (!defined('UPLOAD_ERR_NO_TMP_DIR')) {
    define('UPLOAD_ERR_NO_TMP_DIR', 6);
}

/* since PHP 4.4.0 */
if (!defined('PHP_INT_MAX')) {
    define('PHP_INT_MAX', 0x7fffffff);
}
if (!defined('PHP_INT_SIZE')) {
    define('PHP_INT_SIZE', 4);
}
if (!defined("SORT_LOCALE_STRING")) {
    define("SORT_LOCALE_STRING", 5);
}

/* introduced in PHP 5 */
if (!defined("E_RECOVERABLE_ERROR")) {
    define("E_RECOVERABLE_ERROR", 4096);
}
if (!defined("E_STRICT")) {
    define("E_STRICT", 2048);
}
/* Perl compatible regular expressions */
if (!defined("PREG_NO_ERROR")) {
    define("PREG_NO_ERROR", 0);
}
if (!defined("PREG_INTERNAL_ERROR")) {
    define("PREG_INTERNAL_ERROR", 1);
}
if (!defined("PREG_BACKTRACK_LIMIT_ERROR")) {
    define("PREG_BACKTRACK_LIMIT_ERROR", 2);
}
if (!defined("PREG_RECURSION_LIMIT_ERROR")) {
    define("PREG_RECURSION_LIMIT_ERROR", 3);
}
if (!defined("PREG_BAD_UTF8_ERROR")) {
    define("PREG_BAD_UTF8_ERROR", 4);
}
/* The following applies to count_recursive()
 * introduced in PHP 5
 */
if (!defined("COUNT_NORMAL")) {
    define("COUNT_NORMAL", 0);
}
if (!defined("COUNT_RECURSIVE")) {
    define("COUNT_RECURSIVE", 1);
}

/* introduced in PHP 5.1.0 */
if (!defined('UPLOAD_ERR_CANT_WRITE')) {
    define('UPLOAD_ERR_CANT_WRITE', 7);
}
if (!defined("ENT_COMPAT")) {
   define("ENT_COMPAT", 2);
}
if (!defined("ENT_QUOTES")) {
   define("ENT_QUOTES", 3);
}
if (!defined("ENT_NOQUOTES")) {
   define("ENT_NOQUOTES", 0);
}

/* Missing constants in 5.1, originally appeared in 4.0. */
if (!defined("M_SQRTPI")) {
    define("M_SQRTPI", 1.7724538509055);
}
if (!defined("M_LNPI")) {
    define("M_LNPI", 1.1447298858494);
}
if (!defined("M_EULER")) {
    define("M_EULER", 0.57721566490153);
}
if (!defined("M_SQRT3")) {
    define("M_SQRT3", 1.7320508075689);
}

/* yana framework only */
if (!defined('UPLOAD_ERR_SIZE')) {
    define('UPLOAD_ERR_SIZE', -1);
}
if (!defined('UPLOAD_ERR_FILE_TYPE')) {
    define('UPLOAD_ERR_FILE_TYPE', -2);
}
if (!defined('UPLOAD_ERR_INVALID_TARGET')) {
    define('UPLOAD_ERR_INVALID_TARGET', -4);
}
if (!defined('UPLOAD_ERR_OTHER')) {
    define('UPLOAD_ERR_OTHER', -5);
}


/* The following applies to (among others)
 *
 * FILE_PUT_CONTENTS()
 *
 * introduced in PHP 5
 */
if (!defined('FILE_USE_INCLUDE_PATH')) {
    define('FILE_USE_INCLUDE_PATH', 1);
}
if (!defined("FILE_IGNORE_NEW_LINES")) {
    define("FILE_IGNORE_NEW_LINES", 2);
}
if (!defined("FILE_SKIP_EMPTY_LINES")) {
    define("FILE_SKIP_EMPTY_LINES", 4);
}
if (!defined('FILE_APPEND')) {
    define('FILE_APPEND', 8);
}
if (!defined("FILE_NO_DEFAULT_CONTEXT")) {
    define("FILE_NO_DEFAULT_CONTEXT", 16);
}

/* introduced in PHP 4.0.1 / 5.1.0 */
if (!defined('LOCK_SH')) {
    define('LOCK_SH', 1);
}
if (!defined('LOCK_EX')) {
    define('LOCK_EX', 2);
}
if (!defined('LOCK_UN')) {
    define('LOCK_UN', 3);
}
if (!defined('LOCK_NB')) {
    define('LOCK_NB', 4);
}

/* removed in PHP5.3.0 */
if (!defined("MHASH_CRC32")) {
    define("MHASH_CRC32", 0);
}
if (!defined("MHASH_MD5")) {
    define("MHASH_MD5", 1); /* RFC1321 */
}
if (!defined("MHASH_SHA1")) {
    define("MHASH_SHA1", 2); /* RFC3174 */
}
if (!defined("MHASH_TIGER")) {
    define("MHASH_TIGER", 7);
}
if (!defined("MHASH_MD4")) {
    define("MHASH_MD4", 16); /* RFC1320 */
}
if (!defined("MHASH_SHA256")) {
    define("MHASH_SHA256", 17);
}
if (!defined("MHASH_ADLER32")) {
    define("MHASH_ADLER32", 18);
}

/**#@-*/

/* FUNCTIONS */

if (!function_exists('is_infinite')) {
    /**
     * is_infinite() for PHP < 4.2.0
     *
     * This functions returns true if $float is an infinite number,
     * like log(0) is.
     *
     * @since   2.9.9
     * @param   float  $float
     * @return  bool
     */
    function is_infinite($float)
    {
        /**
         * Input checking
         */
        if (!is_float($float)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Float', gettype($float)), E_USER_WARNING);
            return false;
        }

        $string = (string) $float;
        if ($string === "INF" || $string === "-INF") {
            return true;
        } else {
            return false;
        }
    }
} /* end if */

if (!function_exists('is_nan')) {
    /**
     * is_nan() for PHP < 4.2.0
     *
     * This functions returns true if $float is not a number,
     * like the result of acos(1.01).
     *
     * @since   2.9.9
     * @param   float  $float
     * @return  bool
     */
    function is_nan($float)
    {
        /**
         * Input checking
         */
        if (!is_float($float)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Float', gettype($float)), E_USER_WARNING);
            return false;
        }

        if ( "$float" === "NAN") {
            return true;
        } else {
            return false;
        }
    }
} /* end if */

if (!function_exists('is_finite')) {
    /**
     * is_finite() for PHP < 4.2.0
     *
     * This functions returns true if $float is a finite number.
     * It's the opposite of is_infinite().
     *
     * @since   2.9.9
     * @param   float  $float
     * @return  bool
     */
    function is_finite($f)
    {
        /**
         * Input checking
         */
        if (!is_float($float)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Float', gettype($float)), E_USER_WARNING);
            return false;
        }

        if ("$float" == $float) {
            return true;
        } else {
            return false;
        }
    }
} /* end if */

if (!function_exists('md5_file')) {
    /**
     * md5_file() for PHP < 4.2
     *
     * The function md5_file() was introduced in PHP 4.2
     * It returns the MD5 hash for a file.
     *
     * @since   2.8.8
     * @param   string   $filename
     * @param   bool     $rawOutput
     * @return  string
     */
    function md5_file($filename, $rawOutput = false)
    {
        /**
         * Input checking
         */
        if (!is_string($filename)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($filename)), E_USER_WARNING);
            return false;
        }
        if (!is_bool($rawOutput)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Boolean', gettype($rawOutput)), E_USER_WARNING);
            return false;
        }

        if (!is_file($filename)) {
            return false;
        } else {
            $fileContents = file_get_contents($filename);
            return md5($fileContents, $rawOutput);
        }

    }
} /* end if */

if (!function_exists('array_change_key_case')) {
    /**
     * array_change_key_case() for PHP < 4.2
     *
     * The function array_change_key_case() was introduced in PHP 4.2
     * It returns a the input array with the keys in upper-case or
     * lower-case depending on argument $mode.
     * If $mode is not set it defaults to CASE_LOWER.
     *
     * For a recursive version see {@link Hashtable::changeCase()}.
     *
     * @since   2.8.8
     * @param   array    $array    some array
     * @param   int      $mode     CASE_LOWER = lower-case / CASE_UPPER = upper-case
     * @return  array
     * @see     Hashtable::changeCase()
     */
    function array_change_key_case($array, $mode = CASE_LOWER)
    {
        /**
         * Input checking
         */
        if (!is_array($array)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Array', gettype($array)), E_USER_WARNING);
            return false;
        }
        if (!is_int($mode)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Integer', gettype($mode)), E_USER_WARNING);
            return false;
        }        
        assert('$mode === CASE_LOWER || $mode === CASE_UPPER; /* Invalid argument 2. Expected one of: CASE_LOWER, CASE_UPPER. */');

        if (is_array($array)) {
            foreach ($array as $i => $value)
            {
                unset($array[$i]);
                if (is_string($i)) {
                    if ($mode === CASE_UPPER) {
                        $i = strtoupper($i);
                    } else {
                        $i = strtolower($i);
                    }
                }
                $array[$i] = $value;
            }
        }
        return $array;
    }
} /* end if */

if (!function_exists('floatval')) {
    /**
     * floatval() for PHP < 4.2.0
     *
     * Alias of doubleval()
     *
     * @since   2.8.8
     * @param   mixed    $var
     * @return  float
     */
    function floatval($var)
    {
        return doubleval($var);
    }
} /* end if */

if (!function_exists('file_get_contents')) {
    /**
     * file_get_contents() for PHP < 4.3.0
     *
     * The function file_get_contents() was introduced in PHP 4.3.0.
     * It returns the contents of a file as a string or bool(false)
     * on error.
     *
     * @since   2.8.7
     * @param   string   $filename
     * @param   bool     $useIncludePath
     * @return  string
     */
    function file_get_contents($filename, $useIncludePath = false)
    {
        /**
         * Input checking
         */
        if (!is_string($filename)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($filename)), E_USER_WARNING);
            return false;
        }

        /* Input handling */
        if ($useIncludePath) {
            $useIncludePath = 1;
        } else {
            $useIncludePath = 0;
        }
        $fileContents = file($filename, $useIncludePath);
        if ($fileContents === false) {
            return false;
        } else {
            return implode('', $fileContents);
        }
    }
} /* end if */

if (!function_exists('file_put_contents')) {
    /**
     * file_put_contents() for PHP < 5.0
     *
     * The function file_put_contents() was introduced in PHP 5
     * It writes the contents of $data to a file and returns the
     * number of bytes written to the file or bool(false) on error.
     *
     * Example:
     * <code>
     * file_put_contents('test.dat', array(1,2,3,4), FILE_APPEND | LOCK_EX);
     * </code>
     *
     * @since   2.8.8
     * @param   string    $filename
     * @param   mixed     $data
     * @param   int       $flags
     * @param   resource  $context
     * @return  int
     */
    function file_put_contents($filename, $data , $flags = null, $context = null)
    {
        /**
         * Input checking
         */
        if (!is_string($filename)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($filename)), E_USER_WARNING);
            return false;
        }
        if (!is_array($data) && !is_scalar($data)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Array or scalar', gettype($data)), E_USER_WARNING);
            return false;
        }
        if (!is_int($flags) && !is_null($flags)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 3, 'Integer', gettype($flags)), E_USER_WARNING);
            return false;
        }
        if (!is_resource($context) && !is_null($context)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 4, 'Resource', gettype($context)), E_USER_WARNING);
            return false;
        }

        /* Input handling */
        if (is_array($data)) {
            $data = implode("", $data);
        }
        if (is_null($context)) {
            if ($flags & FILE_APPEND) {
                $handle = fopen($filename, "a");
            } else {
                $handle = fopen($filename, "w+");
            }
        } else {
            $handle = $context;
        }
        if ($handle === false) {
            return false;
        } else {
            if ($flags & LOCK_EX) {
                flock($handle, LOCK_EX);
            }
            $result = fwrite($handle, $data);
            if ($flags & LOCK_EX) {
                flock($handle, LOCK_UN);
            }
            fclose($handle);
        }
        if ($result === false) {
            return false;
        } else {
            return $result;
        }
    }
} /* end if */

if (!function_exists('stripos')) {
    /**
     * stripos() for PHP < 5.0
     *
     * The function stripos() was introduced in PHP 5
     * It returns the position of $needle in the string $haystack.
     *
     * It is case-insensitive.
     *
     * The name can be read as follows:
     * STRing_Ignore-case_POSition (stripos)
     *
     * @since   2.8.8
     * @param   string   $haystack
     * @param   string   $needle
     * @param   int      $offset
     * @return  int
     */
    function stripos($haystack, $needle, $offset = 0)
    {
        /**
         * Input checking
         */
        if (!is_string($haystack)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($haystack)), E_USER_WARNING);
            return false;
        }
        if (!is_string($needle)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($needle)), E_USER_WARNING);
            return false;
        }
        if (!is_int($offset)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 3, 'Integer', gettype($offset)), E_USER_WARNING);
            return false;
        }

        $haystack = strtolower($haystack);
        $needle = strtolower($needle);
        if ($offset !== 0) {
            return strpos($haystack, $needle, $offset);
        } else {
            return strpos($haystack, $needle);
        }
    }
} /* end if */

if (!function_exists('strripos')) {
    /**
     * strripos() for PHP < 5.0
     *
     * The function strripos() was introduced in PHP 5
     * It returns the position of $needle in the string $haystack
     * starting from the end of the string.
     *
     * It is case-insensitive.
     *
     * The name can be read as follows:
     * STRing_Right_Ignore-case_POSition (strripos)
     *
     * @since   2.8.8
     * @param   string   $haystack
     * @param   string   $needle
     * @param   int      $offset
     * @return  int
     */
    function strripos($haystack, $needle, $offset = 0)
    {
        /**
         * Input checking
         */
        if (!is_string($haystack)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($haystack)), E_USER_WARNING);
            return false;
        }
        if (!is_string($needle)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($needle)), E_USER_WARNING);
            return false;
        }
        if (!is_int($offset)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 3, 'Integer', gettype($offset)), E_USER_WARNING);
            return false;
        }

        $haystack = strtolower($haystack);
        $needle   = strtolower($needle);
        if ($offset !== 0) {
            return  strpos($haystack, $needle, $offset);
        } else {
            return  strpos($haystack, $needle);
        }
    }
} /* end if */

if (!function_exists('substr_compare')) {
    /**
     * substr_compare() for PHP < 5.0
     *
     * The function substr_compare() was introduced in PHP 5
     * It does the same as strcmp(), but can be used on substrings.
     *
     * @since   2.8.8
     * @param   string   $mainString
     * @param   string   $string
     * @param   int      $offset
     * @param   int      $length
     * @param   bool     $caseInsensitivity
     * @return  int
     */
    function substr_compare($mainString, $string, $offset, $length = 0, $caseInsensitivity = false)
    {
        /**
         * Input checking
         */
        if (!is_string($mainString)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($mainString)), E_USER_WARNING);
            return false;
        }
        if (!is_string($string)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($string)), E_USER_WARNING);
            return false;
        }
        if (!is_int($offset)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 3, 'Integer', gettype($offset)), E_USER_WARNING);
            return false;
        }
        if (!is_int($length)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 4, 'Integer', gettype($length)), E_USER_WARNING);
            return false;
        }
        if (!is_bool($caseInsensitivity)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 5, 'Boolean', gettype($caseInsensitivity)), E_USER_WARNING);
            return false;
        }

        $mainString = substr($mainString, $offset, $length);
        $string = substr($string, $offset, $length);

        if ($caseInsensitivity === true) {
            return strcasecmp($mainString, $string);
        } else {
            return strcasecmp($mainString, $string);
        }

    }
} /* end if */

if (!function_exists('str_ireplace')) {
    /**
     * str_ireplace() for PHP < 5.0
     *
     * The function str_ireplace() was introduced in PHP 5.
     * Note: this function does not fully support the argument
     * $count, which was added in PHP 5.0.0.
     *
     * Use as follows in PHP 4:
     * <code>
     * $subject = str_ireplace($search, $replace, $subject, &$count);
     * print "found {$count}x $search.\n";
     * </code>
     *
     * @since   2.9.9
     * @param   mixed $search
     * @param   mixed $replace
     * @param   mixed $subject
     * @return  mixed
     */
    function str_ireplace($search, $replace, $subject, $count = 0)
    {
        /**
         * Input checking
         */
        if (is_string($search)) {
            $search = array($search);
        } elseif (!is_array($search)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($search)), E_USER_WARNING);
            return false;
        }
        if (is_string($replace)) {
            $replace = array($replace);
        } elseif (!is_array($replace)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($replace)), E_USER_WARNING);
            return false;
        }
        if (is_string($subject)) {
            $subject = array($subject);
        } elseif (!is_array($subject)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 3, 'String', gettype($subject)), E_USER_WARNING);
            return false;
        }
        if (count($search) !== count($replace)) {
            trigger_error("Number of search values does not match number of replace values", E_USER_WARNING);
            return false;
        }
        
        $count = 0;

        assert('!isset($i); // Cannot redeclare var $i');
        for ($i = 0; $i < count($subject); $i++)
        {
            assert('!isset($j); // Cannot redeclare var $j');
            for ($j = 0; $j < count($search); $j++)
            {
                $searchLength = strlen($search[$j]);
                $replaceLength = strlen($replace[$j]);
                $search[$j] = strtolower($search[$j]);
                $current = strtolower($subject[$i]);
                $pos = 0;
                while (($pos = strpos($current, $search[$j], $pos)) !== false)
                {
                    $count++;
                    $subject[$i] = substr_replace($subject[$i], $replace[$j], $pos, $searchLength);
                    $pos += $searchLength;
                }
                unset($pos);
            }
            unset($j);
        }
        unset($i);

        if (count($subject) === 1) {
            return implode('', $subject);
        } else {
            return $subject;
        }

    }
} /* end if */

if (!function_exists('strpbrk')) {
    /**
     * strpbrk() for PHP < 5.0
     *
     * The function strpbrk() was introduced in PHP 5
     * It searches the string for a list of characters
     * and returns the substring starting at the first
     * match, or bool(false) if no match is found.
     *
     * @since   2.8.8
     * @param   string   $haystack
     * @param   string   $charList
     * @return  string
     */
    function strpbrk($haystack, $charList)
    {
        /**
         * Input checking
         */
        if (!is_string($haystack)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($haystack)), E_USER_WARNING);
            return false;
        }
        if (!is_string($charList)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($charList)), E_USER_WARNING);
            return false;
        }

        if (empty($charList) || empty($haystack)) {
            return false;
        }

        $test = preg_match('/[' . preg_quote($charList, '/') . '].*$/s', $haystack, $match);

        if (!empty($match[0])) {
            return $match[0];
        } else {
            return false;
        }

    }
} /* end if */

if (!function_exists('scandir')) {
    /**
     * scandir() for PHP < 5
     *
     * The function scandir() was introduced in PHP 5
     * It returns a list of files and directories inside
     * a $directory.
     *
     * @since   2.8.8
     * @param   string    $directory      directory name
     * @param   int       $sorting_order  0 = ascending / 1 = descending
     * @param   resource  $context        currently not supported
     * @return  array
     * @see     dirlist()
     */
    function scandir($directory, $sorting_order = 0, $context = null)
    {
        /**
         * Input checking
         */
        if (!is_string($directory)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($directory)), E_USER_WARNING);
            return false;
        } elseif (!is_dir($directory)) {
            trigger_error("The directory '${directory}' does not exist.", E_USER_WARNING);
            return false;
        }

        $contents = dirlist($directory);
        if (!is_array($contents)) {
            return false;
        } else {
            if ($sorting_order === 1) {
                rsort($contents);
            } else {
                sort($contents);
            }
            return $contents;
        }
    }
} /* end if */

if (!function_exists('array_combine')) {
    /**
     * array_combine() for PHP < 5
     *
     * The function array_combine() was introduced in PHP 5
     * It takes a list of keys and a list of values and combines both
     * to a new array.
     *
     * @since   2.8.8
     * @param   array    $keys      list of keys
     * @param   array    $values    list of values
     * @return  array
     */
    function array_combine($keys, $values)
    {
        /**
         * Input checking
         */
        if (!is_array($keys)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Array', gettype($keys)), E_USER_WARNING);
            return false;
        }
        if (!is_array($values)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Array', gettype($values)), E_USER_WARNING);
            return false;
        }
        if (count($keys) !== count($values)) {
            trigger_error('Key-array and value-array must have equal size.', E_USER_WARNING);
            return false;
        }

        if (!empty($keys)) {
            $combinedArray = array();
            while (!empty($keys))
            {
                $combinedArray[array_shift($keys)] = array_shift($values);
            } /* end foreach */
            return $combinedArray;

        } else {
            return false;
        }
    }
} /* end if */

if (!function_exists("json_encode")) {
    /**
     * json_encode() for PHP < 5.0
     *
     * Converts PHP variable or array into a "JSON" (JavaScript object notation) string.
     *
     * Note: this implementation is not comatible with Unicode.
     *
     * Note: this implementation is slow.
     *
     * @author  Mario Salzer
     * @param   mixed  $var  PHP variable/array/object
     * @param   bool   $obj  ignore
     * @return  string
     */
    function json_encode($var, $obj = false)
    {
        /**
         * Input checking
         */
        if (!is_bool($obj)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Boolean', gettype($obj)), E_USER_WARNING);
            return false;
        }

        /* prepare JSON string */
        $json = "";
  
        /* add array entries */
        if (is_array($var) || ($obj = is_object($var))) {

           /* check if array is associative */
           if (!$obj) {
              foreach ((array) $var as $i => $v)
              {
                  if (!is_int($i)) {
                     $obj = 1;
                     break;
                  }
               }
           }

           /* concat invidual entries */
           foreach ((array) $var as $i => $v)
           {
              $json .= ($json ? "," : "") /* comma separators */
              . ($obj ? ("\"$i\":") : "") /* assoc prefix */
              . (json_encode($v)); /* value */
           }

           /* enclose into braces or brackets */
           if ($obj) {
               $json = "{".$json."}";
           } else {
               $json = "[".$json."]";
           }

        /* strings need some care */
        } elseif (is_string($var)) {
              if (!utf8_decode($var)) {
                 $var = utf8_encode($var);
              }
              $var = str_replace(array("\"", "\\", "/", "\b", "\f", "\n", "\r", "\t"), array("\\\"", "\\\\", "\\/", "\\b", "\\f", "\\n", "\\r", "\\t"), $var);
              $json = '"' . $var . '"';
              //@COMPAT: for fully-fully-compliance   $var = preg_replace("/[\000-\037]/", "", $var);
        /* basic types */
        } elseif (is_bool($var)) {
            if ($var) {
                $json = "true";
            } else {
                $json = "false";
            }
        } elseif ($var === null) {
           $json = "null";
        } elseif (is_int($var) || is_float($var)) {
           $json = "$var";
        }
  
        /* something went wrong */
        else {
           trigger_error("Unexpected argument type '" .gettype($var). "'.", E_USER_WARNING);
        }
        
        /* done */
        return($json);
    }
} /* end if */

if (!function_exists("json_decode")) {
    /**
     * json_decode() for PHP < 5.0
     *
     * Parses a JSON (JavaScript object notation) string into a PHP variable
     * (array or object).
     *
     * Note: this implementation is slow.
     *
     * @author  Mario Salzer
     * @param   string  $json     JSON encoded values
     * @param   bool    $assoc    (optional) if outer shell should be decoded as object always
     * @param   mixed   $n        ignore
     * @param   mixed   $state    ignore
     * @param   mixed   $waitfor  ignore
     * @return  mixed
     */
    function json_decode($json, $assoc = false, $n = 0, $state = 0, $waitfor = 0)
    {

        /* result var */
        $val = null;
        static $lang_eq = array("true" => true, "false" => false, "null" => null);
        static $str_eq = array("n" => "\012", "r" => "\015", "\\" => "\\", '"' => '"', "f" => "\f", "b" => "\b", "t" => "\t", "/" => "/");
  
        /* flat char-wise parsing */
        for (/*n*/; $n<strlen($json); /*n*/)
        {
            $c = $json[$n];
        
            /* -= in-string */
            if ($state==='"') {
        
                if ($c == '\\') {
                    $c = $json[++$n];
                    /* simple C escapes */
                    if (isset($str_eq[$c])) {
                        $val .= $str_eq[$c];
                 
                    /* here we transform \uXXXX Unicode (always 4 nibbles) references to UTF-8 */
                    } elseif ($c == "u") {
                        /* read just 16bit (therefore value can't be negative) */
                        $hex = hexdec( substr($json, $n+1, 4) );
                        $n += 4;
                        /* Unicode ranges */
                        if ($hex < 0x80) {    // plain ASCII character
                            $val .= chr($hex);
                        } elseif ($hex < 0x800) {   // 110xxxxx 10xxxxxx
                            $val .= chr(0xC0 + $hex>>6) . chr(0x80 + $hex&63);
                        } elseif ($hex <= 0xFFFF) { // 1110xxxx 10xxxxxx 10xxxxxx
                            $val .= chr(0xE0 + $hex>>12) . chr(0x80 + ($hex>>6)&63) . chr(0x80 + $hex&63);
                        } else {
                            /* other ranges, like 0x1FFFFF=0xF0, 0x3FFFFFF=0xF8 and 0x7FFFFFFF=0xFC do not apply */
                        }
        
                    /* no escape, just a redundant backslash
                     * @COMPAT: we could throw an exception here
                     */
                    } else {
                        $val .= "\\" . $c;
                    }
        
                /* end of string */
                } elseif ($c == '"') {
                   $state = 0;
                /* single character */
                } else { //@COMPAT: specialchars check - but native json doesn't do it?
                   $val .= $c;
                }
        
            /* -> end of sub-call (array/object) */
            } elseif ($waitfor && (strpos($waitfor, $c) !== false)) {
                return array($val, $n);  // return current value and state
        
            /* -= in-array */
            } elseif ($state === ']') {
                list($v, $n) = json_decode($json, 0, $n, 0, ",]");
                $val[] = $v;
                if ($json[$n] == "]") {
                    return array($val, $n);
                }
        
            /* -= in-object */
            } elseif ($state === '}') {
                list($i, $n) = json_decode($json, 0, $n, 0, ":");   // this allowed non-string indicies
                list($v, $n) = json_decode($json, 0, $n+1, 0, ",}");
                $val[$i] = $v;
                if ($json[$n] == "}") {
                    return array($val, $n);
                }
        
            /* looking for next item (0) */
            } else {
        
                /* whitespace */
                if (preg_match("/\s/", $c)) {
                    /* intentionally left blank */
             
                /* string begin */
                } elseif ($c == '"') {
                    $state = '"';
             
                /* object */
                } elseif ($c == "{") {
                    list($val, $n) = json_decode($json, $assoc, $n+1, '}', "}");
                    if ($val && $n && !$assoc) {
                        $obj = new stdClass();
                        foreach ($val as $i => $v)
                        {
                            $obj->{$i} = $v;
                        }
                        $val = $obj;
                        unset($obj);
                    }
             
                /* array */
                } elseif ($c == "[") {
                    list($val, $n) = json_decode($json, $assoc, $n+1, ']', "]");
             
                /* comment */
                } elseif (($c == "/") && ($json[$n+1]=="*")) {
                    // just find end, skip over
                    ($n = strpos($json, "*/", $n+1)) or ($n = strlen($json));
             
                /* numbers */
                } elseif (preg_match("#^(-?\d+(?:\.\d+)?)(?:[eE]([-+]?\d+))?#", substr($json, $n), $uu)) {
                    $val = $uu[1];
                    $n += strlen($uu[0]) - 1;
                    if (strpos($val, ".")) {  /* float */
                        $val = (float)$val;
                    } elseif ($val[0] == "0") {  /* oct */
                        $val = octdec($val);
                    } else {
                        $val = (int)$val;
                    }
                    /* exponent? */
                    if (isset($uu[2])) {
                       $val *= pow(10, (int)$uu[2]);
                    }
        
                /* boolean or null */
                } elseif (preg_match("#^(true|false|null)\b#", substr($json, $n), $uu)) {
                    $val = $lang_eq[$uu[1]];
                    $n += strlen($uu[1]) - 1;
             
                /* parsing error */
                } else {
                    /* PHPs native json_decode() breaks here usually and QUIETLY */
                    trigger_error("json_decode: error parsing '$c' at position $n", E_USER_WARNING);
                    if ($waitfor) {
                        return array(null, 1<<30);
                    } else {
                        return null;
                    }
                }
        
            } /* end if */
            
            /* next char */
            if ($n === null) {
                return null;
            } else {
                $n++;
            }
        } /* end for */

        /* final result */
        return ($val);
    }
} /* end if */

if (!function_exists("fprintf")) {
    /**
     * fprintf() for PHP < 5.0
     *
     * write formatted string to stream/file,
     * arbitrary numer of arguments
     *
     * @param   resource  $stream
     * @param   string    $format  same as of printf()
     * @param   mixed     $args    list of one or more arguments
     * @return  int
     */
    function fprintf($stream, $format /* ... */)
    {
        /**
         * Input checking
         */
        if (!is_resource($stream)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Resource', gettype($stream)), E_USER_WARNING);
            return 0;
        }
        if (!is_string($format)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($format)), E_USER_WARNING);
            return 0;
        }

        $args = func_get_args();
        array_shift($args);

        $string = call_user_func_array("sprintf", $args);
        if (!empty($string)) {
            return fwrite($stream, $string);
        } else {
            return 0;
        }
    }
} /* end if */

if (!function_exists("vfprintf")) {
    /**
     * vfprintf() for PHP < 5.0
     *
     * write formatted string to stream, args array
     *
     * @param   resource  $stream
     * @param   string    $format  same as of printf()
     * @param   array     $args
     * @return  int
     */
    function vfprintf($stream, $format, $args = null)
    {
        /**
         * Input checking
         */
        if (!is_resource($stream)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Resource', gettype($stream)), E_USER_WARNING);
            return 0;
        }
        if (!is_string($format)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($format)), E_USER_WARNING);
            return 0;
        }
        if (!is_array($args) || !is_null($args)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 3, 'Array', gettype($args)), E_USER_WARNING);
            return 0;
        }

        $string = vsprintf($format, $args);
        if (!empty($string)) {
            return fwrite($stream, $string);
        } else {
            return 0;
        }
    }
} /* end if */


if (!function_exists("str_split")) {
    /**
     * str_split() for PHP < 5.0
     *
     * Splits a string in evenly sized chunks.
     * Returns false on error.
     *
     * @param  string  $string        input string
     * @param  int     $split_length  chunk size
     * @return array
     */
    function str_split($string, $split_length = 1)
    {

        /**
         * Input checking
         */
        if (!is_string($string)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($string)), E_USER_WARNING);
            return false;
        }
        if (!is_int($split_length)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Integer', gettype($split_length)), E_USER_WARNING);
            return false;
        }
        /* error: split length to small */
        if ($split_length < 1) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Integer greater 0', gettype($split_length) . "($split_length)"), E_USER_WARNING);
            return false;
        }
  
        $array = array();
        for ($i = 0; $i < strlen($string); $i += $split_length)
        {
            $array[] = substr($string, $i, $split_length);
        }

        return $array;
    }
} /* end if */

if (!function_exists('array_product')) {
    /**
     * array_product() for PHP < 5.1
     *
     * The function array_product() was introduced in PHP 5.1.0RC1
     * It returns the product of the numbers inside the array.
     *
     * If a non-numeric value is encountered, it is converted to an integer
     * using the "intval()" function.
     *
     * Note: unlike the original PHP-function this implementation
     * will issue an E_USER_NOTICE if a non-numeric value is found.
     *
     * This function returns null on error.
     *
     * @since   2.8.8
     * @param   array    $array    list of numbers
     * @return  number
     */
    function array_product($array)
    {
        /**
         * Input checking
         */
        if (!is_array($array)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Array', gettype($array)), E_USER_WARNING);
            return null;
        }

        if (empty($array)) {
            return 0;
        }
        $product = 1;
        foreach ($array as $element)
        {
            if (is_numeric($element)) {
                $product *= $element;
            } else {
                $product *= intval($element);
                trigger_error("The input-array contains a non-numeric value of type '".gettype($element)."'.\n\t\tThe result is undefined.", E_USER_NOTICE);
            }
        } /* end foreach */
        return $product;
    }
} /* end if */

if (!function_exists('property_exists')) {
    /**
     * array_product() for PHP < 5.1.0RC1
     *
     * The function property_exists() was introduced in PHP 5.1.0RC1
     * It returns true if the object or class $class has a property
     * named $property and false otherwise
     *
     * @since   2.8.8
     * @param   mixed    $class     class name or object
     * @param   string   $property  name of the property
     * @return  bool
     */
    function property_exists($class, $property)
    {
        /**
         * Input checking
         */
        if (!is_string($property)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, "2 in ".__FUNCTION__."()", 'String', gettype($property)), E_USER_WARNING);
            return false;
        }

        if (is_object($class)) {
            $class = get_class($class);
        }

        if (is_string($class)) {
            $array = get_class_vars($class);
            if (!is_array($array)) {
                return false;
            } else {
                $array = array_keys($array);
            }
            if (in_array($property, $array)) {
                return true;
            } else {
                return false;
            }
        } else {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, "1 in ".__FUNCTION__."()", 'String or object', gettype($class)), E_USER_WARNING);
            return false;
        }
    }
}

if (!function_exists('http_build_query')) {
    /**
     * http_build_query() for PHP < 5.0
     *
     * The function http_build_query() was introduced in PHP 5
     * It encodes an array to a valid URI - query string.
     *
     * @since   2.9.4
     * @param   array    $formdata
     * @param   string   $numeric_prefix
     * @param   string   $arg_separator
     * @return  string
     */
    function http_build_query($formdata, $numeric_prefix = "", $arg_separator = null)
    {
        /**
         * Input checking
         */
        if (!is_array($formdata)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, "1 in ".__FUNCTION__."()", 'Array', gettype($formdata)), E_USER_WARNING);
            return false;
        }
        if (!is_string($numeric_prefix)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, "2 in ".__FUNCTION__."()", 'String', gettype($numeric_prefix)), E_USER_WARNING);
            return false;
        }

        if (func_num_args() > 2) {
            $index = func_get_arg(2);
        } else {
            $index = false;
        }
        if (is_null($arg_separator)) {
            $sep = ini_get('arg_separator.output');
        } else {
            $sep = $arg_separator;
        }

        $query = "";
        foreach ($formdata as $key => $element)
        {
            if (is_numeric($key)) {
                $key = $numeric_prefix . $key;
            }
            if ($index) {
                $key = "${index}[${key}]";
            }
            if ($query !== '') {
                $query .= $sep;
            }
            if (is_array($element)) {
                $query .= http_build_query($element, $numeric_prefix, $key);
            } else {
                $query .= urlencode( $key ) . "=" . urlencode( $element );
            }
        }
        return $query;
    }
}

if (!function_exists('htmlspecialchars_decode')) {
    /**
     * htmlspecialchars_decode() for PHP < 5.1
     *
     * The function htmlspecialchars_decode() was introduced in PHP 5.1.0RC1.
     * It returns a dequoted string.
     *
     * Symbols are translated as follows:
     * <ul>
     *  <li>'&amp;' = '&'</li>
     *  <li>'&lt;' = '<'</li>
     *  <li>'&gt;' = '>'</li>
     *  <li>'&quot;' = '"' (when ENT_NOQUOTES is not set)</li>
     *  <li>'&#039;' = ''' (only when ENT_QUOTES is set)</li>
     * </ul>
     *
     * @since   2.9.7
     * @param   string    $string
     * @param   int       $quote_style
     * @return  string
     */
    function htmlspecialchars_decode($string, $quote_style = ENT_COMPAT)
    {
        /**
         * Input checking
         */
        if (!is_string($string)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($string)), E_USER_WARNING);
            return false;
        }
        if (!is_int($quote_style)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Integer', gettype($quote_style)), E_USER_WARNING);
            $quote_style = ENT_COMPAT;
        }

        switch (true)
        {
            /**
             * 1) leave both double and single quotes unconverted
             */
            case $quote_style & ENT_NOQUOTES:
                /* intentionally left blank */
            break;

            /**
             * 2) convert both double and single quotes
             */
            case $quote_style & ENT_QUOTES:
                $string = str_replace('&#039;', "'", $string);
            /* fall trough */

            /**
             * 3) convert double-quotes and leave single-quotes alone
             */
            case $quote_style & ENT_COMPAT:
                $string = str_replace('&quot;', '"', $string);
            break;

            /**
             * 4) error: invalid input
             */
            default:
                /* intentionally left blank */
            break;
        } /* end switch */
        
        $string = str_replace('&lt;', '<', $string);
        $string = str_replace('&gt;', '>', $string);
        $string = str_replace('&amp;', '&', $string);

        return $string;
    }
}

if (!function_exists('sys_get_temp_dir')) {
    /**
     * This function is new to PHP 5.
     * Currently no version information available.
     *
     * Will try to return the system's temporary directory,
     * or bool(false) on failure.
     *
     * Note: in some previous releases of PHP this function
     * was named 'php_get_temp_dir()'.
     *
     * This function will try several known settings starting
     * with the system's environment vars, through to PHP's
     * ini settings.
     *
     * Important note: This function MAY NOT return the real
     * temporary directory of the system. It returns SOME
     * temporary directory.
     *
     * Seems to work fine with Windows. Not tested with POSIX.
     *
     * @since   2.9.4
     * @param   mixed    $var
     * @return  string
     */
    function sys_get_temp_dir()
    {
        /* This will work for Server2Go */
        if (isset($_ENV['S2G_TEMP_FOLDER'])) {
            return $_ENV['S2G_TEMP_FOLDER'];

        /* previous name of the same function in an early PHP-release */
        } elseif (function_exists('php_get_temp_dir')) {
            return php_get_temp_dir();

        /* environment vars */
        } elseif (isset($_ENV['TEMP'])) {
            return $_ENV['TEMP'];
        } elseif (isset($_ENV['TMP'])) {
            return $_ENV['TMP'];

        /* ini and config vars */
        } else {
            $tmp = ini_get('temp_dir');
            if (!empty($tmp)) {
                return $tmp;
            }
            $tmp = ini_get('upload_tmp_dir');
            if (!empty($tmp)) {
                return $tmp;
            }
            $tmp = ini_get('session.save_path');
            if (!empty($tmp)) {
                return $tmp;
            }
            $tmp = get_cfg_var('upload_tmp_dir');
            if (!empty($tmp)) {
                return $tmp;
            }
            $tmp = get_cfg_var('session.save_path');
            if (!empty($tmp)) {
                return $tmp;
            }

            /**
             * No value found.
             *
             * Note: that this function does not return
             * a bogus "tmp" directory (which possibly
             * might not even exist) in this case.
             */
            return false;
        }
    }
}
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
