<?php
/**
 * YANA PHP-Framework
 *
 * Constant definitions and basic configuration-file
 *
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package  yana
 * @license  http://www.gnu.org/licenses/gpl.txt
 */

/*
 * When debugging, make sure error reporting is at maximum.
 */
if (error_reporting() > 0) {
    error_reporting(E_ALL);
}

/**
 * Set encoding to UTF-8
 */
if (PHP_VERSION_ID < 50600) {
    iconv_set_encoding('input_encoding', 'UTF-8');
    iconv_set_encoding('output_encoding', 'UTF-8');
    iconv_set_encoding('internal_encoding', 'UTF-8');
} else {
    ini_set('default_charset', 'UTF-8');
}
mb_http_input("UTF-8");
mb_internal_encoding("UTF-8");
mb_http_output("UTF-8");
mb_language('uni');

/**#@+
 * preload common tools
 *
 * @ignore
 */

require_once('includes/toolbox.php');
require_once('includes/php4fallback.php');

/**#@-*/

/**#@+
 * escape method
 *
 * These constants are used by the functions StructureFile::untaintInput()
 * and untaintInput() to identify the method that should be used to escape
 * characters of an input string.
 *
 * See the API-documentation of these functions for details and examples.
 *
 * {@internal
 * Don't use integer 1 as value, to avoid accidental evaluation to boolean true.
 * Don't change the value of YANA_ESCAPE_ALL unless you got a real good reason.
 * }}
 *
 * @see  StructureFile::untaintInput()
 * @see  untaintInput()
 */

if (!defined('YANA_ESCAPE_NONE'))       define('YANA_ESCAPE_NONE',       0);
if (!defined('YANA_ESCAPE_SLASHED'))    define('YANA_ESCAPE_SLASHED',    2);
if (!defined('YANA_ESCAPE_TOKEN'))      define('YANA_ESCAPE_TOKEN',      4);
if (!defined('YANA_ESCAPE_CODED'))      define('YANA_ESCAPE_CODED',      8);
if (!defined('YANA_ESCAPE_LINEBREAK'))  define('YANA_ESCAPE_LINEBREAK',  16);
if (!defined('YANA_ESCAPE_USERTEXT'))   define('YANA_ESCAPE_USERTEXT',   32);
if (!defined('YANA_ESCAPE_ALL'))        define('YANA_ESCAPE_ALL',        PHP_INT_MAX -2);

/**#@-*/
/**#@+
 * counter settings
 *
 * These constants are used by the class Counter to identify wether the visitor's
 * remote address (IP) should be remembered or not, to avoid counting twice when
 * the page reloads in the visitor's browser.
 *
 * @see  Counter
 */

if (!defined('YANA_COUNTER_USE_IP'))     define('YANA_COUNTER_USE_IP',    0);
if (!defined('YANA_COUNTER_IGNORE_IP'))  define('YANA_COUNTER_IGNORE_IP', 1);

/**#@-*/
/**#@+
 * request vars
 *
 * These constants are used to identify the method via which arguments have
 * been sent to the script.
 *
 * @see  Yana::getRequestVar()
 */

if (!defined('YANA_REQUEST')) define('YANA_REQUEST', 0);
if (!defined('YANA_POST'))    define('YANA_POST',    1);
if (!defined('YANA_GET'))     define('YANA_GET',     2);

/**#@-*/
/**#@+
 * directory scanning
 *
 * These constants are used by the functions dirlist() and Dirstream::dirlist().
 * See the documentation of these for details.
 *
 * @see  dirlist()
 * @see  Dirstream::dirlist()
 */

if (!defined('YANA_GET_ALL'))    define('YANA_GET_ALL',   0);
if (!defined('YANA_GET_FILES'))  define('YANA_GET_FILES', 1);
if (!defined('YANA_GET_DIRS'))   define('YANA_GET_DIRS',  2);

/**#@-*/
if (!defined('YANA_VERSION')) {
    /**
     * currently installed version of the Yana Framework
     *
     * Note: you can compare two version strings using the PHP-function
     * version_compare(). See the PHP manual for details.
     */
    define('YANA_VERSION', '3.0.2');
}
if (!defined('YANA_IS_STABLE')) {
    /**
     * this is true, if the current version is stable
     */
    define('YANA_IS_STABLE', true);
}

/**#@+
 * error reporting
 *
 * @see  ErrorUtility::setErrorReporting()
 */
if (!defined('YANA_ERROR_OFF')) {
    /**
     * turn error reporting off
     *
     * Use this in any production environment to prevent information leak to
     * possible attackers.
     */
    define('YANA_ERROR_OFF', 'off');
}
if (!defined('YANA_ERROR_ON')) {
    /**
     * turn error reporting on
     *
     * Use this to debug your scripts.
     */
    define('YANA_ERROR_ON',  'on');
}
if (!defined('YANA_ERROR_LOG')) {
    /**
     * send errors to a log file
     *
     * Write all reported messages to 'cache/error.log'.
     * Use this for testing.
     */
    define('YANA_ERROR_LOG', 'log');
}
/**#@-*/
if (!defined('ENT_FULL')) {
    /**
     * Constant ENT_FULL
     *
     * This adds a new constant ENT_FULL to the php predefined constants
     * ENT_COMPAT=2, ENT_QUOTES=3, ENT_NOQUOTES=0
     *
     * Use of this constant with the method String::encode(), will force
     * ALL characters of the string to be encoded as html entities.
     *
     * See this example:
     * <code>$string->encode('entities', ENT_FULL)</code>
     *
     * {@internal
     * Note:
     * There is no '1' for a good reason. In PHP as in some other
     * languages int(1) is fairly equivalent to bool(true), while
     * string(""), int(0) are equivalent to bool(false).
     * So you might encounter unexpected results when using int(1)
     * in other than an int-context. Not using int(1) is a way
     * (or propably a soft kind of "hack") to evade this.
     *
     * Note:
     * Just in case there may be any additional constants in future
     * there is a good chance they will continue as 4, 5, ...
     * So setting proprietary value ENT_FULL to 10 should'nt collide
     * for now and the near future ;-)
     * }}
     *
     * @see  String::encode()
     */
    define('ENT_FULL', 10);
}

if (!defined('STRING_UNSUPPORTED_ENCRYPTION')) {
    /**
     * Constant UNSUPPORTED_ENCRYPTION
     *
     * This is returned by String::encrypt()
     * when an invalid encryption method is
     * encountered.
     *
     * @see  String
     */
    define('STRING_UNSUPPORTED_ENCRYPTION', 1);
}

/**#@+
 * type identifier
 *
 * These constants are used by the function checkArgumentList()
 * to identify the type of an argument. These can be combined using
 * the bitwise OR operator, like IS_INT|IS_NULL (read as "is integer or null").
 *
 * {@internal
 * To understand it best, don't think of integer as a number here, but as an array
 * of boolean values, which has a fixed size.
 *
 * Any added type needs to have a value double the value of the one before.
 * Or in other words, the sequence translates to 2^1, 2^2, 2^3, aso.
 * Otherwise the behaviour mentioned above is broken, because the bitwise operator
 * won't work anymore.
 *
 * Explanation in detail: This has to do with the binary representation of an
 * integer, where 2 translates to '010', 4 to '100' aso. This means, bitwise 'OR' of
 * 2|4 produces '110' (integer: 6) and as you can see from this, you can easily tell
 * the operands. If the second bit is 1, then integer 2 is one of them, if the third
 * bit is 1, then integer 4 was present, otherwise (if it is 0) you can for sure
 * tell, that there was no operand 4.
 *
 * As a result, 2&6 translates to '010', which is (surprise) integer 2. In fact:
 * "2&$foo" is always either integer 2 or integer 0. You may also interpret this as
 * boolean true and false. This means, the operation can easily be reversed and
 * efficiently used for comparision. It's also the fastest way to do it.
 * That's why this solution is so popular.
 *
 * Nethertheless you should keep in mind, that you cannot have more values then the
 * width in bit of the integer type. So if you use 32 bit integers, this limits you
 * to a maximum of 31 values.
 *
 * You may check the width of the integer type by using this code:
 * <code>print PHP_INT_SIZE * 8;</code>
 * }}
 *
 * @see  checkArgumentList()
 */

if (!defined('IS_INT'))       define('IS_INT',       2);
if (!defined('IS_STRING'))    define('IS_STRING',    4);
if (!defined('IS_FLOAT'))     define('IS_FLOAT',     8);
if (!defined('IS_BOOL'))      define('IS_BOOL',      16);
if (!defined('IS_ARRAY'))     define('IS_ARRAY',     32);
if (!defined('IS_SCALAR'))    define('IS_SCALAR',    64);
if (!defined('IS_NUMERIC'))   define('IS_NUMERIC',   128);
if (!defined('IS_OBJECT'))    define('IS_OBJECT',    256);
if (!defined('IS_RESOURCE'))  define('IS_RESOURCE',  512);
if (!defined('IS_NULL'))      define('IS_NULL',      1024);
if (!defined('IS_MIXED'))     define('IS_MIXED',     2048);

/**#@-*/
/**#@+
 * form generator identifier
 *
 * These constants are used (amongst others) by the function FormCreator::getFormdata()
 * to identify which form a set of input vars is comming from.
 * See the API-documentation of this function for details and examples.
 *
 * {@internal
 * Leave value 0 blank to avoid accidental evaluation to bool(false).
 * }}
 *
 * @see  FormCreator
 */

if (!defined('FORM_VIEW'))        define('FORM_VIEW',       1);
if (!defined('FORM_SEPERATED'))   define('FORM_SEPERATED',  2);
if (!defined('FORM_DETAILS'))     define('FORM_DETAILS',    3);
if (!defined('FORM_EDIT'))        define('FORM_EDIT',       4);
if (!defined('FORM_NEW'))         define('FORM_NEW',        5);
if (!defined('FORM_SEARCH'))      define('FORM_SEARCH',     6);
if (!defined('FORM_ADV_SEARCH'))  define('FORM_ADV_SEARCH', 7);

/**#@-*/

if (!defined('YANA_CDROM')) {
    /**
     * select type of media (online / offline)
     *
     * This should be bool(false) when installed on a webserver.
     * If you want to run yana from a DVD or CD-ROM, this should be set to bool(true).
     *
     * In more detail: This setting defines where to store user settings and files.
     * When false they will be kept in the yana installation directory.
     * When true they will be moved to a temporary directory on a local HDD.
     */
    define('YANA_CDROM', false);
}

if (!defined('YANA_CDROM_DIR')) {
    if (YANA_CDROM === true) {

        /**
         * temporary directory when running on CD-ROM or DVD
         *
         * Here you may specify a path, where files should be stored, when running
         * the YANA framework on a media, where you can't store files.
         *
         * Example:
         * <code>
         * // Try to determine the system's temporary directory automatically, ...
         * define('YANA_CDROM_DIR', sys_get_temp_dir() . "yana/");
         * // or set it to a fixed directory.
         * define('YANA_CDROM_DIR', "c:/temp/yana/");
         * // for Server2Go on CD-ROM you may also use the following ...
         * define('YANA_CDROM_DIR', $_ENV['S2G_TEMP_FOLDER'] . "yana/");
         * </code>
         */
        define('YANA_CDROM_DIR', sys_get_temp_dir() . "yana/");

    } else {
        /**
         * @ignore
         */
        define('YANA_CDROM_DIR', '');
    }
}

if (!defined('YANA_EMBTAG_ALLOW_PHP')) {
    /**
     * activate/deactivate php emb-tag
     *
     * This enables/disables the emb-tag "php", which transfers php code in guestbook
     * or forum posts to a higlighted text representation in HTML.
     * Note that this does NOT execute the code.
     *
     * Under certain circumstances, e.g. where you do not need this feature, you may want
     * to disable it. To do so, set this constant to bool(false). Otherwise it should be
     * set to bool(true).
     */
    define('YANA_EMBTAG_ALLOW_PHP', true);
}

if (!defined('YANA_DB_COMPAT_INSERT')) {
    /**
     * activate/deactivate database compatibility mode on insert() operation
     *
     * Makes the database operation "insert()" behave like in version 2.9.4 and earlier.
     * Set this constant to bool(true) to enable the compatibility mode, or set it to
     * bool(false) to disable it.
     */
    define('YANA_DB_COMPAT_INSERT', false);
}

if (!defined('YANA_CACHE_ACTIVE')) {
    /**
     * activate/deactivate Yana Framework's system cache
     *
     * This constant enables/disables the framework's internal system cache, that
     * accelerates the startup process of the framework in productive environments.
     *
     * You may want to turn this feature off for debugging.
     *
     * Set to bool(true) to enable, or bool(false) to disable the feature.
     * By default this setting is activated and deactivated automatically
     * together with the debugging mode.
     */
    define("YANA_CACHE_ACTIVE", error_reporting() === 0);
}

/**#@+
 * database constants
 *
 * @ignore
 */

if (!defined('YANA_DB_FAILURE'))             define('YANA_DB_FAILURE',            -1);
if (!defined('YANA_DB_NOT_AVAILABLE'))       define('YANA_DB_NOT_AVAILABLE',      0);
if (!defined('YANA_DB_NO_FILE'))             define('YANA_DB_NO_FILE',            1);
if (!defined('YANA_DB_OK'))                  define('YANA_DB_OK',                 2);
if (!defined('YANA_TRIGGER_BEFORE'))         define('YANA_TRIGGER_BEFORE',        2);
if (!defined('YANA_TRIGGER_AFTER'))          define('YANA_TRIGGER_AFTER',         4);
if (!defined('YANA_DB_UNKNOWN'))             define('YANA_DB_UNKNOWN',            0);
if (!defined('YANA_DB_SELECT'))              define('YANA_DB_SELECT',             8);
if (!defined('YANA_DB_UPDATE'))              define('YANA_DB_UPDATE',             16);
if (!defined('YANA_DB_INSERT'))              define('YANA_DB_INSERT',             32);
if (!defined('YANA_DB_DELETE'))              define('YANA_DB_DELETE',             64);
if (!defined('YANA_DB_EXISTS'))              define('YANA_DB_EXISTS',             128);
if (!defined('YANA_DB_LENGTH'))              define('YANA_DB_LENGTH',             256);
if (!defined('YANA_DB_TABLE'))               define('YANA_DB_TABLE',              1);
if (!defined('YANA_DB_ROW'))                 define('YANA_DB_ROW',                2);
if (!defined('YANA_DB_CELL'))                define('YANA_DB_CELL',               3);
if (!defined('YANA_DB_COLUMN'))              define('YANA_DB_COLUMN',             4);
if (!defined('YANA_DB_NONE'))                define('YANA_DB_NONE',               0);
if (!defined('YANA_DB_DELETE_FILE'))         define('YANA_DB_DELETE_FILE',        -1);
if (!defined('YANA_DB_ROW_WILDCARD'))        define('YANA_DB_ROW_WILDCARD',       1);
if (!defined('YANA_DB_COL_WILDCARD'))        define('YANA_DB_COL_WILDCARD',       2);
if (!defined('YANA_DB_TRIGGER'))             define('YANA_DB_TRIGGER',            0);
if (!defined('YANA_DB_CONSTRAINT'))          define('YANA_DB_CONSTRAINT',         1);
if (!defined('YANA_AUTODEQUOTE'))            define('YANA_AUTODEQUOTE',           true);

/**#@-*/
/**#@+
 * regular expressions and word filter constants
 *
 * @ignore
 */

if (!defined('YANA_DB_DELIMITER'))           define('YANA_DB_DELIMITER',          '"');
if (!defined('YANA_UNI_DELIMITER'))          define('YANA_UNI_DELIMITER',         '%');
if (!defined('YANA_UNI_DELIMITER_REGEXP'))   define('YANA_UNI_DELIMITER_REGEXP',  '%');
if (!defined('YANA_LEFT_DELIMITER'))         define('YANA_LEFT_DELIMITER',        '['.YANA_UNI_DELIMITER);
if (!defined('YANA_RIGHT_DELIMITER'))        define('YANA_RIGHT_DELIMITER',       YANA_UNI_DELIMITER.']');
if (!defined('YANA_REG_WORD'))               define('YANA_REG_WORD',              '\$?[\w_\.]');

/**#@-*/
/**#@+
 * configurations and settings
 *
 * @ignore
 */

if (!defined('YANA_SESSION_NAME'))           define('YANA_SESSION_NAME',          'ysid');

if (!defined('YANA_TPL_CACHE'))              define('YANA_TPL_CACHE',             0);
if (!defined('YANA_TPL_CACHE_DIR'))          define('YANA_TPL_CACHE_DIR',         true);
if (!defined('YANA_TPL_FUNCTION'))           define('YANA_TPL_FUNCTION',          0);
if (!defined('YANA_TPL_MODIFIER'))           define('YANA_TPL_MODIFIER',          1);
if (!defined('YANA_TPL_BLOCK'))              define('YANA_TPL_BLOCK',             2);

/**#@-*/
/**#@+
 * constants required for error handling
 *
 * @ignore
 */

if (!defined('YANA_ERROR_WRONG_ARGUMENT'))   define('YANA_ERROR_WRONG_ARGUMENT',  'Wrong argument type for argument %s. %s expected, found \'%s\' instead.');
if (!defined('E_USER_ASSERT'))               define('E_USER_ASSERT',              10);
if (!defined('E_UNKNOWN_ERROR'))             define('E_UNKNOWN_ERROR',            -1);

/**#@-*/
/**#@+
 * The instance-id identifies the current instance of the installation,
 * where multiple instances of the framework are available on the same server.
 *
 * @ignore
 */

if (!defined('YANA_INSTANCE_ID')) {
    if (isset($_SERVER['REMOTE_ADDR'])) {
        define("YANA_INSTANCE_ID", md5( getcwd() . $_SERVER['REMOTE_ADDR']) );
    } else {
        define("YANA_INSTANCE_ID", md5( getcwd() ) );
    }
}

/**#@-*/
/**#@+
 * import external definitions
 *
 * @ignore
 */

/* [SMARTY] */
if (!class_exists('Smarty')) {
    include_once('engine/libs/Smarty.class.php');
}

/* [DATABASE SETTINGS] */

include_once('config/dbconfig.php');

/* [Class-Files] */

require_once('includes/class_utility.php');
require_once('includes/class_object.php');
require_once('includes/class_singleton.php');
require_once('includes/class_stringobject.php');
require_once('includes/class_hashtable.php');
require_once('includes/class_filesystemresource.php');
require_once('includes/class_filereadonly.php');
require_once('includes/class_file.php');
require_once('includes/class_blockfile.php');
require_once('includes/class_floodfile.php');
require_once('includes/class_sml.php');
require_once('includes/class_datfile.php');
require_once('includes/class_dir.php');
require_once('includes/class_vdrive.php');
require_once('includes/class_vdrive_mountpoint.php');
require_once('includes/class_vdrive_dir.php');
require_once('includes/class_vdrive_file.php');
require_once('includes/class_registry.php');
require_once('includes/class_formmailer.php');
require_once('includes/class_executeable.php');
require_once('includes/class_plugin.php');
require_once('includes/class_language.php');
require_once('includes/class_skin.php');
require_once('includes/class_pluginmanager.php');
require_once('includes/class_yana.php');
require_once('includes/class_sessionmanager.php');
require_once('includes/class_dbserver.php');
require_once('includes/class_dbdatatype.php');
require_once('includes/class_dbcreator.php');
require_once('includes/class_dbextractor.php');
require_once('includes/class_dbinfotable.php');
require_once('includes/class_dbinfocolumn.php');
require_once('includes/class_dbstream.php');
require_once('includes/class_dbquery.php');
require_once('includes/class_dbqueryparser.php');
require_once('includes/class_dbtrigger.php');
require_once('includes/class_dbblob.php');
require_once('includes/class_dbstructure.php');
require_once('includes/class_dbmdb2.php');
require_once('includes/class_dbdesigner4.php');
require_once('includes/class_filedb.php');
require_once('includes/class_filedbconnection.php');
require_once('includes/class_filedbindex.php');
require_once('includes/class_filedbresult.php');
require_once('includes/class_smarttemplate.php');
require_once('includes/class_smartview.php');
require_once('includes/class_smartutility.php');
require_once('includes/class_mailer.php');
require_once('includes/class_counter.php');
require_once('includes/class_report.php');
require_once('includes/class_report_message.php');
require_once('includes/class_report_alert.php');
require_once('includes/class_report_error.php');
require_once('includes/class_report_log.php');
require_once('includes/class_report_warning.php');
require_once('includes/class_errorutility.php');
require_once('includes/class_image.php');
require_once('includes/class_image_brush.php');
require_once('includes/class_formcreator.php');
require_once('includes/class_microsummary.php');
require_once('includes/class_rss.php');
require_once('includes/class_rssitem.php');

/* [Deprecated classes and functions] */
require_once('includes/deprecated.php');

/* [Main Class] */

require_once('includes/class_index.php');
/**#@-*/
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
