<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * automated spam protection administration menu
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 *
 * @ignore
 */
class plugin_antispam_admin extends Plugin
{
    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        global $YANA;
        /* do nothing */
        return true;
    }

    /**
     * event handler
     *
     * this function does not expect any arguments
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function get_setup_global_antispam($ARGS)
    {
        /* this function expects no arguments */

        global $YANA;
        $YANA->setVar("WRITEABLE", $YANA->registry->files['config']->isWriteable());
        $YANA->setVar("NEXT_ACTION", 'set_setup_global_antispam');
        return true;
    }

    /**
     * event handler
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function set_setup_global_antispam($ARGS)
    {
        return $GLOBALS['YANA']->handle('set_config_default', $this->_get_arguments($ARGS));
    }

    /**
     * event handler
     *
     * this function does not expect any arguments
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function get_setup_antispam($ARGS)
    {
        /* this function expects no arguments */

        global $YANA;
        $YANA->setVar("WRITEABLE", $YANA->registry->files['config']->isWriteable());
        $YANA->setVar("NEXT_ACTION", 'set_setup_antispam');
        return true;
    }

    /**
     * event handler
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function set_setup_antispam($ARGS)
    {
        return $GLOBALS['YANA']->handle('set_config_profile', $this->_get_arguments($ARGS));
    }

    /**
     * event handler
     *
     * @access  private
     * @param   array  $ARGS  array of params passed to the function
     * @return  array
     * @ignore
     */
    function _get_arguments($ARGS)
    {
        $settings = array();
        if (isset($ARGS['spam/level']) && is_numeric($ARGS['spam/level'])) {
            $ARGS['spam/level'] = (int) $ARGS['spam/level'];

            if (isset($ARGS['spam/words'])) {
                if (is_array($ARGS['spam/words'])) {
                    foreach ($ARGS['spam/words'] as $i => $word)
                    {
                        if ($word != '') {
                            $settings['spam/words'][$i] = htmlentities($word);
                        }
                    }
                    unset($i, $word);
                }
                unset($ARGS['spam/words']);
            }
            if (isset($settings['spam/words'])) {
                $oldWords = $GLOBALS['YANA']->getVar('PROFILE.SPAM.WORDS');
                foreach (array_keys($oldWords) as $word)
                {
                    if (!isset($settings['spam/words'][$word])) {
                        $settings['spam/words'][$word] = null;
                    }
                }
                unset($word);
            }
            switch ($ARGS['spam/level'])
            {
                case 3:
                    $settings['spam/level'] = 3;
                    $settings['spam/permission'] = true;
                    $settings['spam/word_filter'] = true;
                    $settings['spam/captcha'] = true;
                    $settings['spam/header'] = true;
                    $settings['spam/log'] = true;
                    $settings['spam/form_id'] = true;
                break;

                case 2:
                    $settings['spam/level'] = 2;
                    $settings['spam/permission'] = false;
                    $settings['spam/word_filter'] = true;
                    $settings['spam/captcha'] = true;
                    $settings['spam/header'] = true;
                    $settings['spam/log'] = true;
                    $settings['spam/form_id'] = true;
                break;

                case 1:
                    $settings['spam/level'] = 1;
                    $settings['spam/permission'] = false;
                    $settings['spam/word_filter'] = false;
                    $settings['spam/captcha'] = false;
                    $settings['spam/header'] = false;
                    $settings['spam/log'] = false;
                    $settings['spam/form_id'] = true;
                break;

                case 0:
                    $settings['spam/level'] = 0;
                    $settings['spam/permission'] = false;
                    $settings['spam/word_filter'] = false;
                    $settings['spam/captcha'] = false;
                    $settings['spam/header'] = false;
                    $settings['spam/log'] = false;
                    $settings['spam/form_id'] = true;
                break;

                case -1:
                default:
                    foreach (array_keys($ARGS) as $key)
                    {
                        if (strpos($key, 'spam/') === 0) {
                            $settings[$key] = $ARGS[$key];
                        }
                    }
                    $settings['spam/level'] = -1;
                break;

            }
        }
        return $settings;
    }
}
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
