<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * «plugin» class "plugin_blog"
 *
 * @access      public
 * @package     yana
 * @subpackage  plugins
 */
class plugin_blog extends Plugin
{

    /**
     * Connection to data source (API)
     *
     * Is created by a factory function and can be either a connection to "some" database or a text-file,
     * depending on user settings. The correct type will be determined automatically. Take a look at the
     * documentation of class 'DbStream' for details on this API. (all alternative classes implement
     * the same interface, regardless whether the source is a file or a databse)
     *
     * {@internal
     *
     * NOTE:
     * As of PHP 5 the notation "var $database" became deprecated, resulting in an E_STRICT_NOTICE.
     * So you may want to change this to "private $database" for PHP 5. Just be aware that doing so will
     * break backwards compatibility with PHP 4.
     *
     * }}
     *
     * @access  private
     * @var     object serializeableObject  Database-API with Query-Builder (also works with text-files)
     *
     * @ignore
     */
    var $database;

    /**
     * Constructor
     *
     * @access  public
     * @name    plugin_blog::__construct()
     * @param   string  $name  a name for this plugin
     */
    function __construct($name)
    {
        assert('is_string($name);');

        $this->name = $name;
        global $YANA;
        $this->database = $YANA->connect("blog");
    }


/* BEGIN custom Event-handlers */

    /* NOTE:
     * All member-functions stated here act as action handlers
     * (event handlers) and may be called directly in a browser
     * by typing index.php?action=function_name
     *
     * (you may exclude a single function from this behaviour by
     * starting it's name with an underscore '_')
     *
     * While it is not required by default, it is thus highly
     * recommended that you specify all functions within
     * the "INTERFACE"-section of your plugin.config.
     * Refer to the manual for details.
     *
     * For security reasons you may want to override this behaviour
     * to deny access by default if no "INTERFACE"-section AND
     * "INTERFACE.function_name.PERMISSION" entry is provided
     * by setting "DEFAULT.EVENT.PERMISSION" to "-1" in the
     * config/system.config file.
     * Note that this might break up some plugins!
     * As an alternative: setting the value to "1" will require
     * users to be at least registered and logged in to be able
     * to call "unprotected" functions.
     */


/* END custom Event-handlers */

/* BEGIN auto-generated Event-handlers */


    /**
     * provide edit-form
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        read
     * Permission:  75
     * Templates:   BLOG_EDIT_BLOG
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_read_edit_blog()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_read_edit_blog ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;
        $this->_createToc();

        if (class_exists('Microsummary')) {
            Microsummary::publish($this->name);
        }

        /* do something */
        return true;
    }


    /**
     * save changes made in edit-form
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        write
     * Permission:  75
     * Templates:   MESSAGE
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_write_edit_blog()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_write_edit_blog ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;


        $updatedEntries = FormCreator::getFormdata(FORM_EDIT, $this->database->structure, 'blog');

        /* no data has been provided */
        if (empty($updatedEntries)) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        }

        foreach ($updatedEntries as $id => $entry)
        {
            $id = strtolower($id);

            /* before doing anything, check if entry exists */
            if (!$this->database->exists("blog.${id}")) {

                /* error - no such entry */
                $YANA->report(new Warning('INPUT_INVALID'));
                return false;

            /* update the row */
            } else if (!$this->database->update("blog.${id}", $entry)) {
                /* error - unable to perform update - possibly readonly */
                return false;
            }
        } /* end for */
        /* commit changes */
        return $this->database->write();
    }


    /**
     * remove a blog-entry
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        write
     * Permission:  75
     * Templates:   MESSAGE
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_write_delete_blog()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_write_delete_blog ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;

        /* check if user forgot to mark at least 1 row */
        if (empty($ARGS['selected_entries']) || !is_array($ARGS['selected_entries'])) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        }
        /* remove entry from database */
        foreach ($ARGS['selected_entries'] as $id)
        {
            if (!$this->database->remove("blog.${id}")) {
                /* entry does not exist */
                $YANA->report(new Warning('INPUT_INVALID'));
                return false;
            }
        } /* end for */
        /* commit changes */
        return $this->database->write();
    }


    /**
     * provide form to create new blog-entry
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        default
     * Permission:  0
     * Templates:   BLOG_NEW_BLOG
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_default_new_blog()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_default_new_blog ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;
        $this->_createToc();
        if (class_exists('Microsummary')) {
            Microsummary::publish($this->name);
        }
        if (class_exists('RSS')) {
            RSS::publish('blog_rss');
        }

        /* do something */
        return true;
    }

    /**
     * alias of plugin_blog::blog_write_new_blog()
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        write
     * Permission:  0
     * Templates:   MESSAGE
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_add_new_blog()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_add_new_blog ($ARGS)
    {
        return $this->blog_write_new_blog($ARGS);
    }

    /**
     * write new blog-entry to database
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        write
     * Permission:  0
     * Templates:   MESSAGE
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_write_new_blog()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_write_new_blog ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;

        $newEntry = FormCreator::getFormdata(FORM_NEW, $this->database->structure, 'blog');

        /* no data has been provided */
        if (empty($newEntry)) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        }

        /* insert new entry into table */
        if (!$this->database->insert("blog.*", $newEntry)) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        } else if ($this->database->write()) {
            if (class_exists('Microsummary')) {
                Microsummary::set($this->name, strtoupper($this->name) . ', update '.date('d M y G:s', time()));
            }
            return true;
        } else {
            return false;
        }
    }

    /**
     * alias of plugin_blog::blog_write_new_blogcmt()
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        write
     * Permission:  0
     * Templates:   MESSAGE
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_add_new_blogcmt()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_add_new_blogcmt ($ARGS)
    {
        return $this->blog_write_new_blogcmt($ARGS);
    }

    /**
     * write new blog-comment to database
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        write
     * Permission:  0
     * Templates:   MESSAGE
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_write_new_blogcmt()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_write_new_blogcmt ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;

        /* avoid spamming */
        $permission = $YANA->getVar("PERMISSION");
        if (!is_int($permission) || $permission < 1) {
            if (!$YANA->handle("security_check_image", $ARGS)) {
                $YANA->report(new Alert('INPUT_INVALID', 'Security check failed!'));
                $YANA->report(new Log('SPAM: CAPTCHA not solved, entry has not been created.'));
                return false;
            }
        }

        $newEntry = FormCreator::getFormdata(FORM_NEW, $this->database->structure, 'blogcmt');

        /* no data has been provided */
        if (empty($newEntry)) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        }

        /* insert new entry into table */
        if (!$this->database->insert("blogcmt.*", $newEntry)) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        } else {
            return $this->database->write();
        }
    }

    /**
     * update a blog-comment in database
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        write
     * Permission:  75
     * Templates:   MESSAGE
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_write_edit_blogcmt()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_write_edit_blogcmt ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;


        $updatedEntries = FormCreator::getFormdata(FORM_EDIT, $this->database->structure, 'blogcmt');

        /* no data has been provided */
        if (empty($updatedEntries)) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        }

        foreach ($updatedEntries as $id => $entry)
        {
            $id = strtolower($id);

            /* before doing anything, check if entry exists */
            if (!$this->database->exists("blogcmt.${id}")) {

                /* error - no such entry */
                $YANA->report(new Warning('INPUT_INVALID'));
                return false;

            /* update the row */
            } else if (!$this->database->update("blogcmt.${id}", $entry)) {
                /* error - unable to perform update - possibly readonly */
                return false;
            }
        } /* end for */
        /* commit changes */
        return $this->database->write();
    }

    /**
     * remove blog-comment from database
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        write
     * Permission:  75
     * Templates:   MESSAGE
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_write_delete_blogcmt()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_write_delete_blogcmt ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;

        /* check if user forgot to mark at least 1 row */
        if (empty($ARGS['selected_entries']) || !is_array($ARGS['selected_entries'])) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        }
        /* remove entry from database */
        foreach ($ARGS['selected_entries'] as $id)
        {
            if (!$this->database->remove("blogcmt.${id}")) {
                /* entry does not exist */
                $YANA->report(new Warning('INPUT_INVALID'));
                return false;
            }
        } /* end for */
        /* commit changes */
        return $this->database->write();
    }

    /**
     * provide search-form for blog
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        read
     * Permission:  0
     * Templates:   BLOG_SEARCH_BLOG
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_read_search_blog()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_read_search_blog ($ARGS)
    {
        $this->_createToc();
        if (class_exists('Microsummary')) {
            Microsummary::publish($this->name);
        }
        if (class_exists('RSS')) {
            RSS::publish('blog_rss');
        }
    }


    /**
     * view block entries (simple form)
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        read
     * Permission:  0
     * Templates:   BLOG_READ_BLOG
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_read_read_blog()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_read_read_blog ($ARGS)
    {
        $this->_createToc();
        if (class_exists('Microsummary')) {
            Microsummary::publish($this->name);
        }
        if (class_exists('RSS')) {
            RSS::publish('blog_rss');
        }
        return true;
    }

    /**
     * view block entries and comments
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        read
     * Permission:  0
     * Templates:   BLOG_READ_SEPERATED_BLOG
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_read_read_seperated_blog()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_read_read_seperated_blog ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;
        $this->_createToc();

        if (class_exists('Microsummary')) {
            Microsummary::publish($this->name);
        }
        if (class_exists('RSS')) {
            RSS::publish('blog_rss');
        }

        if (isset($ARGS['blog_id']) && is_numeric($ARGS['blog_id'])) {
            $id = (int) $ARGS['blog_id'];
            $YANA->setVar('WHERE', "blog_id=$id");
            $YANA->setVar('ENTRIES', 1);
            $YANA->setVar('BLOG_ID', $id);

            $query = array(
            'key' => 'blogcmt',
            'order_by' => 'blogcmt_created',
            'desc' => true,
            'where' => array( array( 'blog_id', $id, '=' ) )
            );
            $rows = $this->database->get($query);
            $YANA->setVar('BLOGCMTS', $rows);

        } else {
            if (isset($ARGS['entries']) && is_numeric($ARGS['entries']) && $ARGS['entries'] > 0) {
                $entries = (int) $ARGS['entries'];
                $YANA->setVar('ENTRIES', "$entries");
            } else if (isset($_SESSION['entries']['blog'])) {
                $entries = (int) $_SESSION['entries']['blog'];
                $YANA->setVar('ENTRIES', "$entries");
            }
            if (isset($ARGS['page']) && is_numeric($ARGS['page']) && $ARGS['page'] > -1) {
                $page = (int) $ARGS['page'];
                $YANA->setVar('PAGE', $page);
            } else if (isset($_SESSION['page']['blog'])) {
                $page = (int) $_SESSION['page']['blog'];
                $YANA->setVar('PAGE', "$page");
            }

            $YANA->setVar('WHERE', '');

            /* get blog id (for comments) */
            if (isset($entries) && $entries === 1 && isset($page)) {

                /* get blog id */
                $id = $this->database->get('blog.*.blog_id', array(), array(), $page, 1);
                if (is_array($id) && count($id) === 1) {
                    $id = array_pop($id);
                }
                if (is_int($id)) {
                    $YANA->setVar('BLOG_ID', $id);
                }

                /* get comments */
                $query = array(
                'key' => 'blogcmt',
                'order_by' => 'blogcmt_created',
                'desc' => true,
                'where' => array( array( 'blog_id', $id, '=' ) )
                );
                $rows = $this->database->get($query);
                $YANA->setVar('BLOGCMTS', $rows);
            }
        }
        return true;
    }

    /**
     * list of latest blog entries
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        read
     * Permission:  0
     * Templates:   BLOG_LIST
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_list()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_list ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;
        $this->_createToc();

        if (class_exists('Microsummary')) {
            Microsummary::publish($this->name);
        }
        if (class_exists('RSS')) {
            RSS::publish('blog_rss');
        }

        /* get entries from database */
        $query = array(
        'key' => 'blog.*.blog_title',
        'order_by' => 'blog_created',
        'desc' => true,
        'limit' => 50
        );
        $rows = $this->database->get($query);
        $YANA->setVar('BLOG', $rows);
        return true;
    }

    /**
     * Edit comments
     *
     * Type:        read
     * Permission:  0
     * Templates:   BLOG_READ_EDIT_BLOGCMT
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_read_edit_blogcmt()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_read_edit_blogcmt ($ARGS)
    {
        $this->_createToc();
        return true;
    }

    /**
     * produce RSS-feed
     *
     * returns bool(true) on success and bool(false) on error
     *
     * Type:        read
     * Permission:  0
     * Templates:   NULL
     *
     * @access  public
     * @return  bool
     * @name    plugin_blog::blog_rss()
     * @param   array  $ARGS  array of params passed to the function
     */
    function blog_rss ($ARGS)
    {
        if (!class_exists('RSS')) {
            trigger_error("RSS wrapper class not found. Unable to proceed.", E_USER_ERROR);
            return false;
        } else {
            global $YANA;
            /* get entries from database */
            $query = array(
            'key' => 'blog.*',
            'order_by' => 'blog_created',
            'desc' => true,
            'limit' => 10
            );
            $rows = $this->database->get($query);
            /* create RSS feed */
            $rss = new RSS();
            /* set description */
            $YANA->language->read('blog');
            $rss->description = $YANA->getVar('LANGUAGE.RSS_DESCRIPTION');
            foreach ($rows as $row)
            {
                $item = new RSSitem();
                /* 1) process title */
                $item->title = $row['BLOG_TITLE'];
                /* 2) process link */
                $id = $row['BLOG_ID'];
                $item->link = SmartUtility::url("action=blog_read_read_seperated_blog&blog_id=$id", true);
                $item->link = str_replace(session_name()."=".session_id(), '', $item->link);
                /* 3) process description */
                $item->description = $row['BLOG_TEXT'];
                $item->description = SmartUtility::embeddedTags($item->description);
                $item->description = SmartUtility::smilies($item->description);
                $item->description = strip_tags($item->description);
                if (strlen($item->description) > 500) {
                    $item->description = substr($item->description, 0, 496).' ...';
                }
                /* 4) process pubDate */
                $item->pubDate = $row['BLOG_CREATED'];
                if (is_numeric($item->pubDate)) {
                    $item->pubDate = date('r', $item->pubDate);
                }
                $rss->addItem($item);
            } /* end foreach */
            print utf8_encode($rss->toString());
            exit(0);
            return true;
        }
    }

/* END auto-generated Event-handlers */

/* BEGIN auto-generated functions */

    /**
     * create table of contents
     *
     * @static
     * @access  private
     * @name    plugin_blog::_createToc()
     * @param   array   $input  associative array containing user form data
     *
     * @ignore
     */
    function _createToc()
    {
        global $YANA;

        $toc = array();
        $permission = $YANA->getVar('PERMISSION');
        $YANA->language->read('blog');
        $toc[SmartUtility::url("action=blog_list", true)] = 'Index';
        if ($permission >= 75) {
            $toc[$YANA->getVar('LANGUAGE.MENU.EDIT')] = array(
                SmartUtility::url("action=blog_default_new_blog", true) => $YANA->getVar('LANGUAGE.BLOG.NEW'),
                SmartUtility::url("action=blog_read_edit_blog", true) => $YANA->getVar('LANGUAGE.BLOG.EDIT'),
                SmartUtility::url("action=blog_read_edit_blogcmt", true) => $YANA->getVar('LANGUAGE.BLOG.EDITCMTS')
            );
        } elseif ($permission >= 30) {
            $toc[$YANA->getVar('LANGUAGE.MENU.EDIT')] = array(
                SmartUtility::url("action=blog_default_new_blog", true) => $YANA->getVar('LANGUAGE.BLOG.NEW')
            );
        }
        $toc[$YANA->getVar('LANGUAGE.MENU.VIEW')] = array(
            SmartUtility::url("action=blog_read_search_blog", true) => $YANA->getVar('LANGUAGE.BLOG.SEARCH'),
            SmartUtility::url("action=blog_read_read_seperated_blog", true) => $YANA->getVar('LANGUAGE.BLOG.VIEW'),
            SmartUtility::url("action=blog_list", true) => $YANA->getVar('LANGUAGE.BLOG.LIST')
        );
        if ($permission >= 30) {
            $toc[SmartUtility::url("action=about&target=blog&type=plugin", true)] = $YANA->getVar('LANGUAGE.ABOUT');
        }
        $YANA->setVar('BLOG_TOC', $toc);
    }

    /* NOTE:
     * By convention you can hide any member function from the plugin manager
     * by using an underscore '_' as the first character of the function's name.
     * This results in: any such function (event) can not be triggered directly
     * by the user through pointing the "action" parameter in the url to that
     * function name, like: index.php?action=function_name
     *
     * (Using underscores is a common PHP4 notation for "private")
     *
     * This also means, if for some reason you need to create a public event handler
     * whose name somehow has to begin with an underscore, you need to declare this
     * within the class' default event handler.
     */

    /**
     * remove tags from user form data
     *
     * @static
     * @access  private
     * @name    plugin_blog::_untaintInput()
     * @param   array   $input  associative array containing user form data
     *
     * @ignore
     */
    function _untaintInput(&$input)
    {
        if (is_array($input)) {
            foreach ($input as $a => $b)
            {
                $input[$a] = untaintInput($b, "string", 0, YANA_ESCAPE_USERTEXT);
            }
        }
    }

/* END auto-generated functions */

}
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
