<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * default library
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * Default library for common functions
 *
 * This plugin is important. It provides functionality
 * that might be usefull for other plugins.
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_default_library extends Plugin
{
    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function _default($event, $ARGS)
    {
        /* check input data */
        assert('is_string($event);');
        assert('is_array($ARGS);');
        settype($event, "string");
        settype($ARGS, "array");
        $event = strtolower($event);
        global $YANA;

        /* BEGIN visitor-counter */
        $my_counter = new counter();
        if (empty($ARGS['id'])) {
            $id = 'default';
        } else {
            $id = $ARGS['id'];
        }
        $YANA->setVar('VISITOR_COUNT', $my_counter->count($id));
        /* END visitor-counter */
        return true;
    }

    /**
     * Create graphical visitor counter
     *
     * Note: This function ends the program.
     *
     * parameters taken:
     *
     * <ul>
     * <li> string id </li>
     * <li> string target  </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function graphic_counter ($ARGS)
    {
        global $YANA;
        if (!empty($ARGS['id'])) {
            $my_counter = new counter();
            $count = $my_counter->count($ARGS['id']);
        } else {
            $my_counter = new counter();
            $count = $my_counter->count('default');
        }

        settype($ARGS['target'], 'int');

        $skinDir = $YANA->getVar("SKINDIR")."default/counter/".$ARGS['target']."/";
        if (!is_dir($skinDir)) {
            $skinDir = $YANA->getVar("SKINDIR")."default/counter/0/";
        }
        $myImage = new Image($skinDir."hintergrund.jpg", 'jpg');
        $myImageValues = Image::getSize($skinDir."hintergrund.jpg");
        $previousImageWidths = 0;

        if (file_exists($skinDir."blank.png")) {
            $imageValues = Image::getSize($skinDir."blank.png");
            $previousImageWidths += $imageValues[0];
        }

        for ($i = 0; $i < strlen("$count"); $i++)
        {

            $filename = $skinDir.substr("$count", strlen("$count")-$i-1, 1).".png";
            $imageValues = Image::getSize($filename);
            $previousImageWidths += $imageValues[0];
            if (($myImageValues[0]-$previousImageWidths) < 0 || ($myImageValues[1]-$imageValues[1]) < 0) {
               unset($myImage);
               $myImage = new Image();
               $myImage->resize(325,55);
               $myImage->drawString("[ERROR] image size exceeds maximum",                                              5,  0);
               $myImage->drawString("or value to big to be displayed",                                                 5, 10);
               $myImage->drawString("1) check size of counter \"".substr("$count", strlen("$count")-$i-1, 1).".png\"", 5, 20);
               $myImage->drawString("2) check dimensions of file \"hintergrund.png\"",                                 5, 30);
               $myImage->drawString("   are big enough for the content.",                                              5, 40);
               $myImage->outputToScreen();
               die;
            };
            $myImage->copyRegion(new Image($filename, 'png'), 0, 0, null, null, ($myImageValues[0]-$previousImageWidths), (int) floor(($myImageValues[1]-$imageValues[1])/2));
            unset($image);

        }

        $myImage->outputToScreen();
        die();
    }

    /**
     * Show Sitemap
     *
     * this function does not expect any arguments
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function sitemap ($ARGS)
    {
        /* this function expects no arguments */
        global $YANA;
        $id = $YANA->getVar('ID');
        if (!empty($id)) {
            $my_counter = new counter();
            $YANA->setVar('VISITOR_COUNT', $my_counter->count($id));
        } else {
            $my_counter = new counter();
            $YANA->setVar('VISITOR_COUNT', $my_counter->count('default'));
        }
        $YANA->view->setFunction(YANA_TPL_FUNCTION, 'sitemap', array(__CLASS__, '_sitemap'));
        return true;
    }

    /**
     * «smarty function» sitemap
     *
     * @access  public
     * @static
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     */
    function _sitemap($params, &$smarty)
    {
        global $YANA;
        $result = "<ul>\n";
        $dir = $YANA->plugins->getPluginDir();
        foreach ($YANA->plugins->getStartmenuList() as $plugin)
        {
            $action = $plugin['ACTION'];
            $image = $plugin['IMAGE'];
            $title = $plugin['TITLE'];
            if (empty($image)) {
                $result .= "\t<li>";
            } else {
                $result .= "\t<li style=\"list-style-image: url('${dir}${image}')\">";
            }
            $result .= "<a href=" . SmartUtility::href("action=${action}") . ">${title}</a></li>\n";
        }
        $result .= "</ul>\n";
        $vars = array('LANGUAGE' => $smarty->get_template_vars('LANGUAGE'));
        $result = SmartUtility::fillTemplate($result, $vars);
        return $result;
    }

    /**
     * clear server's template-cache
     *
     * this function does not expect any arguments
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function clear_server_cache ($ARGS)
    {
        /* this function expects no arguments */
        smartTemplate::clearCache();
        return true;
    }

    /**
     * create preview message
     *
     * Note: this function ends the program
     *
     * parameters taken:
     *
     * <ul>
     * <li> string eintraege    text to be previewed </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function preview ($ARGS)
    {
        global $YANA;
        if (!isset($ARGS['eintraege'])) {
            trigger_error("Required argument 'eintraege' has not been given.", E_USER_ERROR);
            die();
        } else if (!is_string($ARGS['eintraege'])) {
            trigger_error("Wrong argument type for argument 'eintraege'. String expected, found ".gettype($ARGS['eintraege'])." instead.", E_USER_ERROR);
            die();
        } else {
            assert('is_string($ARGS["eintraege"]);');
            $ARGS['eintraege'] = untaintInput($ARGS['eintraege'], "text", 0, YANA_ESCAPE_USERTEXT, true);
            $ARGS['eintraege'] = SmartUtility::smilies($ARGS['eintraege']);
            $ARGS['eintraege'] = SmartUtility::embeddedTags($ARGS['eintraege']);
            if (isset($ARGS['is_ajax_request']) && $ARGS['is_ajax_request'] == 1) {
                exit($ARGS['eintraege']);
            } else {
                $doc = new SmartTemplate($YANA->skin->getFile('BLANK'));
                $doc->insert('INSERT_CONTENT_HERE', '<div style="overflow: hidden; height: 100%;">' . $ARGS['eintraege'] . '</div>');
                exit($doc->get());
            }
        }
    }

    /**
     *  check for correct syntax of embedded tags in all textarea fields
     *
     * Note: this function ends the program
     *
     * parameters taken:
     *
     * <ul>
     * <li> array text  text to be checked </li>
     * </ul>
     *
     * This functions produces a XML output, containing an array,
     * where index "error" contains an error number and index
     * "text" contains a description.
     *
     * If no error is found, the result is empty.
     *
     * List of error codes:
     * <ul>
     *  <li> 0 = no error </li>
     *  <li> 1 = unexpected start tag </li>
     *  <li> 2 = unknown or unhandled tag </li>
     *  <li> 3 = unexpected end tag </li>
     *  <li> 4 = missing an expected tag </li>
     *  <li> 5 = missing end tag </li>
     *  <li> 6 = unexpected line break </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function chkembtags ($ARGS)
    {
        global $YANA;
        $tags = 'b|i|u|emp|h|c|small|big|code|hide|php|mark|color|mail|img|url';
        $userTags = $YANA->getVar('PROFILE.EMBTAG');
        if (is_array($userTags) && !empty($userTags)) {
            $tags .= '|' . strtolower(implode('|', array_keys($userTags)));
        }
        /* get strings */
        $langError      = ($YANA->getVar('LANGUAGE.TAGS.JS.ERR'));
        $langChar       = ($YANA->getVar('LANGUAGE.TAGS.JS.CHAR'));
        $langEndTag     = ($YANA->getVar('LANGUAGE.TAGS.JS.END'));
        $langUnknownTag = ($YANA->getVar('LANGUAGE.TAGS.JS.UNKNOWN'));
        $langUnexpTag   = ($YANA->getVar('LANGUAGE.TAGS.JS.UNEXP'));
        $langExpTag     = ($YANA->getVar('LANGUAGE.TAGS.JS.EXP'));
        $langUnclTag    = ($YANA->getVar('LANGUAGE.TAGS.JS.UNCL'));
        $langBr         = ($YANA->getVar('LANGUAGE.TAGS.JS.BR'));
        $langProceed    = ($YANA->getVar('LANGUAGE.TAGS.JS.PROCEED'));

        /* check request */
        if (isset($ARGS['text']) && is_array($ARGS['text'])) {
            foreach ($ARGS['text'] as $index => $val)
            {
                if (!is_string($val)) {
                    continue;
                }
                $val = preg_replace('/\[w?br\]/', '', $val);
                $isTag = false;
                $isEndTag = false;
                $buffer = "";
                $tagList = array();
                $top = -1;
    
                for ($i = 0; $i < strlen($val); $i++)
                {
                    $c = $val[$i];
                    switch ($c)
                    {
                        /**
                         * begin of tag
                         */
                        case '[':
                            if ($isTag) {
                                header('Content-type: text/xml');
                                header('Charset: utf-8');
                                exit(XMLencode(array('error' => 1, 'text' => $langError . " " . $langChar . " " . $i . ".\n" . $langEndTag . " [" . $buffer . "].\n" . $langProceed)));
                            }
                            $isTag = true;
                        break;
                        /**
                         * end of tag
                         */
                        case '=':
                        case ']':
                            if ($isTag) {
                                if (!preg_match('/' . $tags . '/', $buffer)) {
                                    header('Content-type: text/xml');
                                    header('Charset: utf-8');
                                    exit(XMLencode(array('error' => 2, 'text' => $langError . " " . $langChar . " " . $i . ".\n" . $langUnknownTag . " [" . $buffer . "].\n" . $langProceed)));
                                } elseif ($isEndTag) {
                                    if ($top < 0) {
                                        header('Content-type: text/xml');
                                        header('Charset: utf-8');
                                        exit(XMLencode(array('error' => 3, 'text' => $langError . " " . $langChar . " " . $i . ".\n" . $langEndTag . " [/" . $buffer . "]. " . $langUnexpTag . ".\n" . $langProceed)));
                                    } elseif ($buffer == $tagList[$top]) {
                                        unset($tagList[$top--]);
                                    } else {
                                        header('Content-type: text/xml');
                                        header('Charset: utf-8');
                                        exit(XMLencode(array('error' => 4, 'text' => $langError . " " . $langChar . " " . $i . ".\n" . $langEndTag . " [/" . $buffer . "]. " . $langExpTag . " [/" . $tagList[$top] . "].\n" . $langProceed)));
                                    }
                                    $isEndTag = false;
                                } else {
                                    $tagList[++$top] = $buffer;
                                }
                                $isTag = false;
                            }
                            $buffer = "";
                        break;
                        /**
                         * is end tag
                         */
                        case '/':
                            if ($isTag) {
                                $isEndTag = true;
                            }
                        break;
                        /**
                         * character
                         */
                        default:
                            if ($isTag) {
                                $buffer .= $c;
                            }
                        break;
                    }
                }
                if ($top > -1) {
                    header('Content-type: text/xml');
                    header('Charset: utf-8');
                    exit(XMLencode(array('error' => 5, 'text' => $langError . ".\n" . $langUnclTag . ": [" . implode('], [', $tagList) . "].\n" . $langProceed)));
                }
                if (preg_match('/^\s*(\[br\]|\n)/s', $ARGS['text'][$index])) {
                    header('Content-type: text/xml');
                    header('Charset: utf-8');
                    exit(XMLencode(array('error' => 6, 'text' => $langError . ".\n" . $langBr . ".\n" . $langProceed)));
                }
            }
        }
        exit(0);
    }

    /**
     * create CAPTCHA-image
     *
     * parameters taken:
     *
     * <ul>
     * <li> int security_image_index    index of image to display </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function security_get_image ($ARGS)
    {
        global $YANA;
        $imagesrc = $YANA->getVar("CONFIGDIR") . "security_image/security_image" . rand(0, 9) . ".png";
        $file = $YANA->plugins->get($this->name, 'SECURITY_IMAGE');

        if (!$file->exists()) {
            $file->create();
        }
        $file->read();
        if (!$file->isEmpty()) {
            $contents = $file->get(0);
        } else {
            $contents = array();
        }

        if (!isset($contents['TIME']) || $contents['TIME'] < time() || $contents['MAX_TIME'] < time()) {
            $contents = array();
            $contents['MAX_TIME'] = time() + 10000;
            for ($i=1;$i<10;$i++)
            {
                $contents["_$i"] = "";
                for ($j=0;$j<5;$j++)
                {
                    $letter = "";
                    while (empty($letter) || strtolower($letter) == 'o' || $letter == '0' || $letter == 'l' || $letter == 'i')
                    {
                        switch (rand(0,2))
                        {
                            case 0:
                                $letter = chr(rand(65,90));
                            break;
                            case 1:
                                $letter = chr(rand(48,57));
                            break;
                            case 2:
                                $letter = chr(rand(97,122));
                            break;
                        }
                    }
                    $contents["_$i"] .= $letter;
                }
            }
        }
        $contents['TIME'] = time() + 1200;
        $file->remove(0);
        $file->insert($contents);
        $file->write();

        if (!isset($ARGS['security_image_index']) || $ARGS['security_image_index'] < 1 || $ARGS['security_image_index'] > 9) {
            $text =& $contents['_1'];
        } else {
            $text =& $contents['_'.$ARGS['security_image_index']];
        }

        $image = new Image($imagesrc, 'png');
        for ($i = 0; $i < 5; $i++)
        {
            $image->drawString($text[$i], 4+($i*9)+rand(0,1), 1+rand(-1,1), $image->getColor(40+rand(-30,60),40+rand(-30,60),40+rand(-30,60)), 5);
        }        
        $image->outputToScreen();
        die();
    }

    /**
     * test if a string matches the corresponding CAPTCHA
     *
     * parameters taken:
     *
     * <ul>
     * <li> int security_image_index    index of image to display </li>
     * <li> string security_image       text to compare with CAPTCHA </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function security_check_image ($ARGS)
    {
        global $YANA;

        $permission = $YANA->getVar("PERMISSION");
        if (is_int($permission) && $permission > 0) {
            return true;
        }

        $file = $YANA->plugins->get($this->name, 'SECURITY_IMAGE');
        $file->read();
        if (!$file->isEmpty()) {
            $contents = $file->get(0);
        } else {
            $contents = array();
        }

        if ($contents['MAX_TIME'] < time()) {
            return false;
        } else {
            if (!isset($ARGS['security_image_index']) || $ARGS['security_image_index'] < 1 || $ARGS['security_image_index'] > 9) {
                $text =& $contents['_1'];
            } else {
                $text =& $contents['_'.$ARGS['security_image_index']];
            }
            if (!empty($text) && ($ARGS['security_image'] == $text)) {
                return true;
            } else {
                return false;
            }
        }
    }

    /**
     * create a microsummary for a page
     *
     * Technical notes:
     *
     * parameters taken:
     *
     * <ul>
     * <li> string id    identifies summary to get </li>
     * </ul>
     *
     * Note: this function terminates program execution.
     *
     * Semantic description:
     *
     * "Microsummaries" are a Firefox 2.0 feature that allows users
     * to create dynamic bookmark titles that automatically updates
     * when the content of the bookmarked page changes.
     *
     * Have a look at what microsummaries can be:
     * <ul>
     *   <li> the numbers of downloads of a file on a download site </li>
     *   <li> the latest news on a news page </li>
     *   <li> current number of unread e-mail in the inbox of a webmail service </li>
     *   <li> current total of donations to a project </li>
     *   <li> the date of latest updates on a database </li>
     *   <li> the latest submission to a guestbook or forum </li>
     *   <li> the number of visitors currently online in a chat room </li>
     *   <li> the latest stock values aso. </li>
     * </ul>
     *
     * Examples of usage:
     * <ol>
     *  <li> Setting a microsummary from a plugin:
     *       <code>Microsummary::set($this->name, 'Summary text');</code>
     *  </li>
     *  <li> Retrieving a microsummary in a plugin:
     *       <code>$microsummary = Microsummary::get($this->name);</code>
     *  </li>
     *  <li> To indicate that a microsummary exists for your plugin
     *       add this as the last line in your plugin constructor
     *       <code>Microsummary::publish($this->name);</code>
     *  </li>
     *  <li> Calling a microsummary from a browser:
     *       <code>index.php?action=get_microsummary&target=guestbook</code>
     *       (where 'guestbook' is the name of the plugin)
     *  </li>
     * </ol>
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function get_microsummary ($ARGS)
    {
        if (empty($ARGS['target']) || !is_string($ARGS['target'])) {
            print 'Error: illegal request';
            exit (1);
        } else {
            $microsummary = Microsummary::get($ARGS['target']);
            if (!empty($microsummary)) {
                print $microsummary;
                exit(0);
            } else {
                print 'No summary available';
                exit(1);
            }
        }
    }

    /**
     * create an image or download a file from a source
     *
     * This function is called by form-generator
     *
     * Note: by default, access to this function is limited
     * to user with security level 100 (maximum).
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function download_file ($ARGS)
    {
        global $YANA;

        $source = DbBlob::getFileId();

        if ($source === false) {
            /* error - file does not exist */
            return false;
        } else {

            /* download a file */
            if (preg_match('/\.gz$/', $source)) {

                $i = 0;

                $gz = gzopen($source, 'r');
                while (!gzeof($gz))
                {
                    $buffer = gzgets($gz, 4096);
                    switch ($i)
                    {
                        case 0:
                            if (preg_match('/^[\w\.\d\-\_]+$/s', $buffer) && !headers_sent()) {
                                header("Content-Disposition: attachment; filename=${buffer}");
                            }
                        break;

                        case 1:
                            if (is_numeric($buffer) && !headers_sent()) {
                                header("Content-Length: ${buffer}");
                            }
                        break;

                        case 2:
                            if (preg_match('/^\w+\/[\w-]+$/s', $buffer) && !headers_sent()) {
                                header("Content-type: ${buffer}");
                            }
                        break;

                        default:
                            print $buffer;
                        break;

                    }
                    $i++;
                }
                gzclose($gz);

            /* download an image */
            } else {
                $image = new Image($source);
                $image->outputToScreen();
            }
            exit(0);
        }
    }

}
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
