<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * IP block setup
 *
 * This implements basic setup functions for the IP block plugin.
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_ipblock_admin extends Plugin
{

    /**#@+
     * @ignore
     * @access  private
     */

    /** @var BlockFile */ var $whitelist;
    /** @var BlockFile */ var $blacklist;
    /** @var Dir */ var $dir;

    /**#@-*/

    /**
     * Constructor
     *
     * @param string $name a name for this plugin
     * @access public
     * @ignore
     */
    function __construct($name)
    {
        settype($name,"string");
        global $YANA;

        $this->name      = $name;
        $this->whitelist = $YANA->plugins->get($this->name,"IPWHITELIST");
        $this->blacklist = $YANA->plugins->get($this->name,"IPBLACKLIST");
        $this->dir       = $YANA->plugins->get($this->name,"BLOCKDIR");
    }

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        settype($event, "string");
        settype($ARGS, "array");

        if ($this->blacklist && $this->whitelist) {
            if ($this->whitelist->exists()) {
                $this->whitelist->read();
            }
            if ($this->blacklist->exists()) {
                $this->blacklist->read();
            }

            global $YANA;

            switch ($event) {

                case "get_block": case "get_root_block":
                    $YANA->setVar("BLACKLIST", $this->blacklist->get());
                    $YANA->setVar("WHITELIST", $this->whitelist->get());
                    return true;
                break;

                case "set_block":
                    $this->blacklist = new blockfile($this->dir->getPath().$ARGS['id'].".blacklist");
                    $this->whitelist = new blockfile($this->dir->getPath().$ARGS['id'].".whitelist");

                    if ($this->whitelist->exists()) {
                        $this->whitelist->read();
                    }
                    if ($this->blacklist->exists()) {
                        $this->blacklist->read();
                    }

                    return $this->_set($ARGS);
                break;

                case "set_root_block":
                    return $this->_set($ARGS);
                break;

                default:
                    /* intentionally left blank */
                break;
            } /* end switch */
        }
        return true;
    }

    /**
     * _untaintInput
     *
     * @access  private
     * @param   array  $ARGS
     * @return  bool
     * @ignore
     */
    function _set($ARGS)
    {
         settype($ARGS, "array");
         global $YANA;

         if (!$this->whitelist->exists() && !$this->whitelist->create()) {
             return false;
         } else  if (!$this->blacklist->exists() && !$this->blacklist->create()) {
             return false;
         } else if ((!$this->blacklist->set($ARGS['blacklist']) || !$this->blacklist->write()) ||
             (!$this->whitelist->set($ARGS['whitelist']) || !$this->whitelist->write()))
         {
             return false;
         } else {
             return true;
         }
    }

}
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
