<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * Software Developement Kit
 *
 * This implements a code generator utility
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_sdk extends Plugin
{

    /**
     * List of DBMS ids and names
     *
     * @access  private
     * @var     array
     * @ignore
     */
    var $listOfDBMS = array(
        'mysql'  => 'MySQL',
        'db2'    => 'DB2',
        'dbase'  => 'DBase',
        'fbsql'  => 'Frontbase',
        'ibase'  => 'Interbase',
        'ifx'    => 'Informix',
        'access' => 'MS-Access',
        'mssql'  => 'MS-SQL',
        'oci8'   => 'Oracle',
        'pgsql'  => 'PostgreSQL',
        'sybase' => 'Sybase'
    );

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        global $YANA;
        /* do nothing */
        return true;
    }

    /**
     * Create form
     *
     * Type: read
     * Permission: 100
     * Templates:  sdk_read_form
     *
     * @param array $ARGS array of params passed to the function
     * @return bool
     * @ignore
     */
    function sdk_read_form ($ARGS)
    {
        global $YANA;
        $dir = $YANA->plugins->get($this->name, 'LOGOS');
        $dir->read();
        $YANA->setVar('FILES', $dir->get());
        $YANA->setVar('IMG_SRC', $dir->getPath());
        $config = $YANA->plugins->get($this->name, 'CONFIGFILE');
        $config->read();
        $YANA->setVar('PLUGIN', $config->get());

        $YANA->setVar('LIST_OF_DBMS', $this->listOfDBMS);
    }

    /**
     * create plugin
     *
     * Type: write
     * Permission: 100
     * Templates: MESSAGE
     *
     * @param array $ARGS array of params passed to the function
     * @return bool
     * @ignore
     */
    function sdk_write_plugin ($ARGS)
    {
        global $YANA;

        $this->_createBackup($ARGS);

        /* Initialize vars */
        $INFO             = array();
        $DRIVE            = array();
        $CUSTOM_INTERFACE = array();
        $AUTO_INTERFACE   = array();
        $INTERFACE        = array();
        $COPY             = array();
        $DATABASE         = array();
        $SKIN             = array();

        /* Check user input */
        $is_valid = true;

        /* NAME */
        if (!is_string($ARGS['name']) || strlen($ARGS['name']) > 15 || !preg_match('/^[\d\w-_ äüöß\(\)]+$/si', $ARGS['name'])) {
            assert('!isset($error); // Cannot redeclare var $error');
            $error = array(
                'FIELD' => 'NAME',
                'VALUE' => print_r($ARGS['name'], true),
                'VALID' => 'a-z, 0-9, -, _, ß, ä, ö, ü, " "'
            );
            $YANA->report(new ErrorMessage("INVALID_CHARACTERS", $error));
            unset($error);
            /* Can't continue without valid id */
            return false;
        } else {
            $INFO['NAME'] = $ARGS['name'];
            $INFO['ID']   = preg_replace('/[^\d\w_]/', '_', strtolower($INFO['NAME']));
            unset($ARGS['name']);
        }

        /* load files */
        $dir = $YANA->plugins->get($this->name, 'LOGOS');        
        $plgDir  = new Dir($YANA->getVar('PLUGINDIR') . $INFO['ID'] . '/');
        $sknDir  = new Dir($YANA->getVar('SKINDIR') . 'default/' . $INFO['ID'] . '/');
        $sknFile = new SML($YANA->getVar('SKINDIR') . 'default/'.$INFO['ID'] . '.config', CASE_UPPER);
        $plgFile = new SML($YANA->getVar('PLUGINDIR') . $INFO['ID'] . '.config', CASE_UPPER);
        $drvFile = new SML($plgDir->getPath() . '_drive.config', CASE_UPPER);
        $phpFile = new File($plgDir->getPath() . 'plugin.php');
        $apiFile = new File($sknDir->getPath() . 'api.js');

        /* GRAFIK */
        if (!is_string($ARGS['grafik']) || !preg_match('/^[\d\w-_\.]+\.(gif|jpg|png)$/si', $ARGS['grafik'])) {
            assert('!isset($error); // Cannot redeclare var $error');
            $error = array(
                'FIELD' => 'LOGO',
                'VALUE' => print_r($ARGS['grafik'], true),
                'VALID' => 'file.gif, file.jpg, file.png'
            );
            $YANA->report(new ErrorMessage("INVALID_SYNTAX", $error));
            unset($error);
            $is_valid = false;
        } else {
            $endung = '.'.preg_replace('/^.*\./', '', $ARGS['grafik']);

            /* copy preview image */
            $i = count($COPY);
            $COPY[$i]['src'] = $dir->getPath() . $ARGS['grafik'];
            $COPY[$i]['dest'] = $plgDir->getPath() . 'preview' . $endung;

            /* copy icon */
            $iconDir = $YANA->plugins->get($this->name, 'ICONS');
            $i = count($COPY);
            $COPY[$i]['src'] = $iconDir->getPath().preg_replace('/(.*)\..*/', "\\1", $ARGS['grafik']).'.png';
            $COPY[$i]['dest'] = $plgDir->getPath().'icon.png';

            $INFO['LOGO'] = '%PLUGINDIR%/' . $INFO['ID'] . '/' . 'preview' . $endung;
            $INFO['IMAGE'] = '%PLUGINDIR%/' . $INFO['ID'] . '/' . 'icon.png';
            unset($ARGS['grafik'], $iconDir, $endung, $i);
        }

        /* PARENT */
        if (!is_string($ARGS['parent']) || !preg_match('/^[\d\w-_]*$/si', $ARGS['parent'])) {
            assert('!isset($error); // Cannot redeclare var $error');
            $error = array(
                'FIELD' => 'PARENT',
                'VALUE' => print_r($ARGS['parent'], true),
                'VALID' => 'a-z, 0-9, -, _'
            );
            $YANA->report(new ErrorMessage("INVALID_CHARACTERS", $error));
            unset($error);
            $is_valid = false;
        } else {
            if (!empty($ARGS['parent'])) {
                $INFO['PARENT'] = strip_tags(nl2br($ARGS['parent']));
            }
            unset($ARGS['parent']);
        }

        /* PACKAGE */
        if (!is_string($ARGS['package']) || !preg_match('/^[\d\w-_]*$/si', $ARGS['package'])) {
            assert('!isset($error); // Cannot redeclare var $error');
            $error = array(
                'FIELD' => 'PACKAGE',
                'VALUE' => print_r($ARGS['package'], true),
                'VALID' => 'a-z, 0-9, -, _'
            );
            $YANA->report(new ErrorMessage("INVALID_CHARACTERS", $error));
            unset($error);
            $is_valid = false;
        } else {
            if (!empty($ARGS['package'])) {
                $INFO['PACKAGE'] = strip_tags(nl2br($ARGS['package']));
            }
            unset($ARGS['package']);
        }

        /* TYPE */
        if (!is_string($ARGS['type']) || !preg_match('/^(primary|default|config|read|write|security)$/si', $ARGS['type'])) {
            assert('!isset($error); // Cannot redeclare var $error');
            $error = array(
                'FIELD' => 'TYPE',
                'VALUE' => print_r($ARGS['type'], true),
                'VALID' => 'primary, default, config, read, write, security'
            );
            $YANA->report(new ErrorMessage("INVALID_SYNTAX", $error));
            unset($error);
            $is_valid = false;
        } else {
            $INFO['TYPE'] = strip_tags(nl2br($ARGS['type']));
            unset($ARGS['type']);
        }

        /* PRIORITY */
        if (!is_string($ARGS['priority']) || !is_numeric($ARGS['priority']) || (int) $ARGS['priority'] != (string) $ARGS['priority'] || $ARGS['priority'] < -1 || $ARGS['priority'] > 3) {
            assert('!isset($error); // Cannot redeclare var $error');
            $error = array(
                'FIELD' => 'PRIORITY',
                'VALUE' => print_r($ARGS['priority'], true),
                'VALID' => '-1 - +3'
            );
            $YANA->report(new ErrorMessage("INVALID_SYNTAX", $error));
            unset($error);
            $is_valid = false;
        } else {
            $INFO['PRIORITY'] = (int) $ARGS['priority'];
            unset($ARGS['priority']);
        }

        /* AUTOR */
        if (!is_string($ARGS['autor'])) {
            $is_valid = false;
        } else {
            if (!empty($ARGS['autor'])) {
                $INFO['AUTHOR'] = strip_tags(nl2br($ARGS['autor']));
            }
            unset($ARGS['autor']);
        }

        /* KONTAKT */
        if (!is_string($ARGS['kontakt'])) {
            $is_valid = false;
        } else {
            if (!empty($ARGS['kontakt'])) {
                $INFO['CONTACT'] = strip_tags(nl2br($ARGS['kontakt']));
            }
            unset($ARGS['kontakt']);
        }


        /* DESCRIPTION */
        if (!is_string($ARGS['beschreibung'])) {
            $is_valid = false;
        } else {
            if (!empty($ARGS['beschreibung'])) {
                $INFO['DESCRIPTION'] = str_replace("\n",'[br]',strip_tags($ARGS['beschreibung']));
            }
            unset($ARGS['beschreibung']);
        }

        /* UPDATE */
        if (!is_string($ARGS['update'])) {
            $is_valid = false;
        } else {
            if (!empty($ARGS['update'])) {
                $INFO['UPDATE'] = strip_tags(nl2br($ARGS['update']));
            }
            unset($ARGS['update']);
        }

        /* if user input is invalid, exit with error */
        if ($is_valid === false) {
            $YANA->report(new ErrorMessage("INPUT_INVALID"));
            return false;
        }

        if (!empty($ARGS['overwrite'])) {
            if ($plgDir->exists()) {
                $plgDir->delete(true);
            }
            if ($sknDir->exists()) {
                $sknDir->delete(true);
            }
            if ($sknFile->exists()) {
                $sknFile->delete();
            }
            if ($plgFile->exists()) {
                $plgFile->delete();
            }
        } elseif ($plgFile->exists() || $plgDir->exists()) {
            $YANA->report(new ErrorMessage("ALLREADY_EXISTS", array('ID' => $INFO['ID'])));
            return false;
        }
        unset($ARGS['overwrite']);

        /* CREATE FILES */
        $plgFile->create();
        $plgDir->create(777);
        $drvFile->create();
        $phpFile->create();
        $sknDir->create(663);
        $apiFile->create();

        { /* sql_file */

            assert('!isset($dbms); // Cannot redeclare $dbms');
            foreach (array_keys($this->listOfDBMS) as $dbms)
            {
                if (!empty($_FILES[$dbms]['tmp_name'])) {
                    if (preg_match('/^\S+\.sql$/s', $_FILES[$dbms]['name'])) {
                        $installDirectory = $YANA->registry->get(strtoupper($dbms));
                        if (!is_object($installDirectory)) {
                            $YANA->report( new Log("Registry error: there is no registered install directory named '${dbms}'. The uploaded SQL-file could not be saved."));
                            continue;
                        }
                        if (!$installDirectory->exists()) {
                            $installDirectory->create(663);
                        }
                        if (!is_dir($installDirectory->getPath())) {
                            $YANA->report( new Log("Unable to create the directory '${dbms}'. The uploaded SQL-file could not be saved."));
                            continue;
                        }
                        move_uploaded_file($_FILES[$dbms]['tmp_name'], $installDirectory->getPath().$INFO['ID'].'.sql');
                        unset($ARGS[$dbms]);
                    } else {
                        $YANA->report( new Alert('INPUT_INVALID') );
                        return false;
                    }
                }
            } /* end foreach */

            unset($dbms);

        }

        { /* CONFIG_TEMPLATE */

            if ($ARGS['config_template'] == 'true') {

                /* create setup entry */
                $INFO['SETUP'] = array();
                $INFO['SETUP'][0] = array();
                $INFO['SETUP'][0]['ACTION'] = $INFO['ID'].'_config_setup';
                $INFO['SETUP'][0]['TITLE']  = $INFO['NAME'];
            }
            unset($ARGS['config_template']);

        }

        if (!empty($_FILES['sourcefile']['tmp_name'])) { /* source_file */

            move_uploaded_file($_FILES['sourcefile']['tmp_name'], $YANA->getVar('DBDIR') . $INFO['ID'] . '.config');
            unset($ARGS['sourcefile']);

            $dbsFile = new DbStructure($YANA->getVar('DBDIR') . $INFO['ID'] . '.config');
            $dbsFile->read();

        }

        { /* INTERFACE */

            if ($ARGS['interface']) {
                $raw_data = explode("\n", $ARGS['interface']);

                for ($i = 0; $i < count($raw_data); $i++)
                {
                    $raw_data[$i] = explode(",",$raw_data[$i]);
                    $j = $raw_data[$i][0];
                    if (!empty($j)) {
                        if (isset($CUSTOM_INTERFACE[$j])) {
                            trigger_error("The action with the name '${j}' has been found multiple time. Previous occurences are overwritten.", E_USER_WARNING);
                        }
                        $CUSTOM_INTERFACE[$j] = array();
                        $CUSTOM_INTERFACE[$j]['TYPE'] = $raw_data[$i][1];
                        $CUSTOM_INTERFACE[$j]['MODE'] = $raw_data[$i][2];
                        if(!empty($raw_data[$i][3])) {
                            $CUSTOM_INTERFACE[$j]['TEMPLATE'] = $raw_data[$i][3];
                        }
                        if(!empty($raw_data[$i][4])) {
                            $CUSTOM_INTERFACE[$j]['INSERT'] = $raw_data[$i][4];
                        }
                        $CUSTOM_INTERFACE[$j]['PERMISSION'] = (int) $raw_data[$i][5];
                    }
                } /* end for */
            }
            unset($ARGS['interface']);
            unset($ARGS['interface_mode']);
        }

        { /* create virtual drive */

            $DRIVE['MAIN'] = array();
            $DRIVE['MAIN']['TYPE']      = 'executeable';
            $DRIVE['MAIN']['REQUIRED']  = 'readable';
            $DRIVE['MAIN']['AUTOMOUNT'] = 'yes';
            $DRIVE['MAIN']['SRC']       = 'plugin.php';

        }

        { /* create GUI and corresponding Interface entries for each table */

            assert('is_array($CUSTOM_INTERFACE); /* unexpected result: $CUSTOM_INTERFACE is not an array*/');
            assert('is_array($INFO); /* unexpected result: $INFO is not an array*/');
            $INPUT = Hashtable::merge($INFO, array('INTERFACE' => $CUSTOM_INTERFACE));
            $HTML_MENU = array();
            $HTML_MENU['table_list'] = array();
            $HTML_MENU['event_list'] = array();

            if (isset($dbsFile) && !$dbsFile->isEmpty()) {
                /*
                 * preprocessor
                 */
                assert('!isset($table); /* Cannot redeclare variable $table. */');
                foreach ($dbsFile->getTables() as $table)
                {
                    assert('!isset($foreignKey); /* Cannot redeclare variable $foreignKey. */');
                    assert('!isset($foreignTable); /* Cannot redeclare variable $foreignTable. */');
                    assert('!isset($primaryKey); /* Cannot redeclare variable $primaryKey. */');
                    assert('!isset($action); /* Cannot redeclare variable $action. */');
                    foreach ($dbsFile->getForeignKeys($table) as $foreignKey => $foreignTable)
                    {
                        $primaryKey = $dbsFile->getPrimaryKey($foreignTable);
                        /*
                         * link on foreign key to primary key
                         */
                        if (! $dbsFile->getAction($table, $foreignKey, 'edit')) {
                            $action = $this->_createActionName($INFO['ID'], 'read', 'edit', $foreignTable);
                            $dbsFile->setAction($table, $foreignKey, $action, 'edit');
                            if (!isset($AUTO_INTERFACE[$action]['REFERENCE'])) {
                                $AUTO_INTERFACE[$action]['REFERENCE'] = array();
                            }
                            $AUTO_INTERFACE[$action]['REFERENCE'][] = array($foreignTable, $primaryKey, $foreignKey);
                        }
                        if (! $dbsFile->getAction($table, $foreignKey, 'select')) {
                            $action = $this->_createActionName($INFO['ID'], 'read', 'read', $foreignTable);
                            $dbsFile->setAction($table, $foreignKey, $action, 'select');
                            if (!isset($AUTO_INTERFACE[$action]['REFERENCE'])) {
                                $AUTO_INTERFACE[$action]['REFERENCE'] = array();
                            }
                            $AUTO_INTERFACE[$action]['REFERENCE'][] = array($foreignTable, $primaryKey, $foreignKey);
                        }
                        /*
                         * link on primary key to foreign key
                         */
                        if (! $dbsFile->getAction($foreignTable, $primaryKey, 'edit')) {
                            $action = $this->_createActionName($INFO['ID'], 'read', 'edit', $table);
                            $dbsFile->setAction($foreignTable, $primaryKey, $action, 'edit');
                            if (!isset($AUTO_INTERFACE[$action]['REFERENCE'])) {
                                $AUTO_INTERFACE[$action]['REFERENCE'] = array();
                            }
                            $AUTO_INTERFACE[$action]['REFERENCE'][] = array($table, $foreignKey, $primaryKey);
                        }
                        if (! $dbsFile->getAction($foreignTable, $primaryKey, 'select')) {
                            $action = $this->_createActionName($INFO['ID'], 'read', 'read', $table);
                            $dbsFile->setAction($foreignTable, $primaryKey, $action, 'select');
                            if (!isset($AUTO_INTERFACE[$action]['REFERENCE'])) {
                                $AUTO_INTERFACE[$action]['REFERENCE'] = array();
                            }
                            $AUTO_INTERFACE[$action]['REFERENCE'][] = array($table, $foreignKey, $primaryKey);
                        }
                    }
                    unset($action, $foreignKey, $foreignTable, $primaryKey);
                }
                unset($table);
                $dbsFile->write();

                assert('!isset($table); /* Cannot redeclare variable $table. */');
                $DATABASE = $dbsFile->get('*');
                foreach ($DATABASE['TABLES'] as $table => $CURRENT)
                {
                    $table = strtolower($table);
                    $HTML_MENU['table_list'][] = $table;
                    $DB_SRC = array('DATABASE' => $CURRENT['CONTENT'], 'TABLE' => $table, 'PRIMARY_KEY' => $CURRENT['PRIMARY_KEY']);

                    { /* Edit */
                        $current_action = $this->_create('gui_edit.html', $INFO['ID'], 'read', 'edit', $INPUT, $DB_SRC, 75, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        $this->_create('', $INFO['ID'], 'write', 'edit', $INPUT, $DB_SRC, 75, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 75, 'table_name' => $table, 'name' => 'edit'));
                    }

                    { /* Delete */
                        $this->_create('', $INFO['ID'], 'write', 'delete', $INPUT, $DB_SRC, 75, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                    }

                    { /* New */
                        $current_action = $this->_create('gui_new.html', $INFO['ID'], 'default', 'new', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        $this->_create('', $INFO['ID'], 'write', 'new', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $table, 'name' => 'new'));
                    }

                    { /* Search */
                        $current_action = $this->_create('gui_search.html', $INFO['ID'], 'read', 'search', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $table, 'name' => 'search'));
                    }

                    { /* View */
                        $current_action = $this->_create('gui_view.html', $INFO['ID'], 'read', 'read', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $table, 'name' => 'view'));
                    }

                } /* end foreach */
                unset($table);
            } /* end if */

        }

        { /* create index file */

            $tplDir = $YANA->plugins->get($this->name, 'TEMPLATES');
            $tplDir = $tplDir->getPath();
            $idxTemplate = new SmartTemplate($tplDir.'gui_index.html');
            $idxTemplate->insert('*', $HTML_MENU);
            if (isset($dbsFile)) {
                $idxTemplate->insert('dbStructure', $dbsFile);
            } else {
                $idxTemplate->insert('dbStructure', false);
            }
            $idxTemplate->insert('plugin_id', $INFO['ID']);
            $IDX = $idxTemplate->get();

        }

        { /* create class skeleton */

            $classFile = $YANA->plugins->get($this->name, 'CLASS');
            $phpTemplate = new SmartTemplate($classFile->getPath());
            $phpTemplate->insert('*', $INFO);
            $phpTemplate->insert('INTERFACE', $CUSTOM_INTERFACE);
            $phpTemplate->insert('AUTO_INTERFACE', $AUTO_INTERFACE);
            if (isset($dbsFile)) {
                $phpTemplate->insert('dbStructure', $dbsFile);
            } else {
                $phpTemplate->insert('dbStructure', false);
            }
            $PHP = $phpTemplate->get();

        }

        { /* create AJAX-Yana bridge */

            $jsFile = $YANA->plugins->get($this->name, 'JSAPI');
            $apiTemplate = new SmartTemplate($jsFile->getPath());
            $apiTemplate->insert('*', $INFO);
            $apiTemplate->insert('CLASS', 'Api' . str_replace(' ', '', ucwords(preg_replace('/_/', ' ', $INFO['ID']))));
            $apiTemplate->insert('INTERFACE', $CUSTOM_INTERFACE);
            $apiTemplate->insert('AUTO_INTERFACE', $AUTO_INTERFACE);
            $API = $apiTemplate->get();

        }

        { /* copy files */

            foreach ($COPY as $k => $v)
            {
                if (!copy($v['src'], $v['dest'])) {
                    $YANA->report(new ErrorMessage('NOT_WRITEABLE', array('FILE'=>$v['dest'])));
                    return false;
                }
            }

        }

        { /* write files */

            assert('is_array($CUSTOM_INTERFACE); /* unexpected result: $CUSTOM_INTERFACE is not an array */');
            assert('is_array($INTERFACE);        /* unexpected result: $INTERFACE is not an array        */');
            $INTERFACE = Hashtable::merge($CUSTOM_INTERFACE, $INTERFACE);
            $plgFile->set(array('INFO' => $INFO, 'INTERFACE' => $INTERFACE));
            if (!$plgFile->write()) {
                $YANA->report(new ErrorMessage('NOT_WRITEABLE', array('FILE'=>$plgFile->getPath())));
                return false;
            }

            $drvFile->set($DRIVE);
            if (!$drvFile->write()) {
                $YANA->report(new ErrorMessage('NOT_WRITEABLE', array('FILE'=>$drvFile->getPath())));
                return false;
            }

            $phpFile->insert("<?php\n".$PHP."\n?>");
            if (!$phpFile->write()) {
                $YANA->report(new ErrorMessage('NOT_WRITEABLE', array('FILE'=>$phpFile->getPath())));
                return false;
            }

            $apiFile->insert($API);
            if (!$apiFile->write()) {
                $YANA->report(new ErrorMessage('NOT_WRITEABLE', array('FILE'=>$apiFile->getPath())));
                return false;
            }

        }

        { /* create Skin file */

            $sknFile->create();
            $sknFile->set($SKIN);
            if (!$sknFile->write()) {
                $YANA->report(new ErrorMessage('NOT_WRITEABLE', array('FILE'=>$sknFile->getPath())));
                return false;
            }


        }

        { /* create Index file */

            $idxFile = new File($sknDir->getPath().'index.html');
            $idxFile->create();
            $idxFile->insert($IDX);
            if (!$idxFile->write()) {
                $YANA->report(new ErrorMessage('NOT_WRITEABLE', array('FILE'=>$idxFile->getPath())));
                return false;
            }

        }

        /* load new plugin */
        $YANA->handle('refresh_pluginlist');

        return true;

    }

    /**
     * @access private
     * @ignore
     */
    function _create ($src, $id, $type, $name, &$INPUT, &$DATABASE, $mode = 0, &$sknDir, &$AUTO_INTERFACE, &$SKIN, &$INTERFACE, &$INFO)
    {
        global $YANA;
        $tplDir   = $YANA->plugins->get($this->name, 'TEMPLATES');
        $tplDir   = $tplDir->getPath();
        $action   = $this->_createActionName($id,$type,$name,$DATABASE['TABLE']);
        $skinName = strtoupper($id.'_'.$name."_".$DATABASE['TABLE']);
        $dest     = strtolower($action.'.html');

        if (!empty($src)) {

            { /* create GUI File */
                $guiTemplate = new SmartTemplate("$tplDir$src");
                $guiTemplate->insert('*', $INPUT);
                $guiTemplate->insert('*', $DATABASE);
                $guiTemplate->insert('ACTION', $action);
                $guiTemplate->insert('ON_NEW', $id."_write_new_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_EDIT', $id."_write_edit_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_DELETE', $id."_write_delete_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_SEARCH', $id."_read_search_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_DOWNLOAD', $id."_".$DATABASE['TABLE']."_download");
                /* add download action to interface */
                $INTERFACE[$id."_".$DATABASE['TABLE']."_download"] = array(
                    'TYPE' => 'read',
                    'MODE' => 0,
                    'PERMISSION' => 100,
                    'TEMPLATE' => 'NULL'
                );
                $html =  $guiTemplate->get();
                $file = new File($sknDir->getPath().$dest);
                if (!$file->exists()) {
                    $file->create();
                    $file->insert($html);
                    if (!$file->write()) {
                        global $YANA;
                        $YANA->report(new ErrorMessage("FILE_NOT_CREATED", array('FILE' => $file->getPath())));
                        $YANA->exitTo('sdk_read_form');
                        return false;
                    }
                }
            }

            { /* create Interface Entry */
                $INTERFACE[$action]['TYPE'] = $type;
                $INTERFACE[$action]['MODE'] = 0;
                $INTERFACE[$action]['INSERT'] = $skinName;
                if ($mode > 0) {
                    $INTERFACE[$action]['PERMISSION'] = $mode;
                } else {
                    $INTERFACE[$action]['PERMISSION'] = 0;
                }
                $INTERFACE[$action]['ONSUCCESS'] = array();
                $INTERFACE[$action]['ONERROR'] = array();
                $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $action;
                $INTERFACE[$action]['ONERROR']['GOTO'] = $action;
            }

            { /* create Skin Entry */
                $SKIN[$skinName] = array(
                'FILE' => "${id}/${dest}",
                'LANGUAGE' => array(),
                'STYLE' => array(),
                'SCRIPT' => array("ajax" => "scripts/ajax.js", "api" => "${id}/api.js")
                );
            }

            { /* create Root entry */
                if (($type == "default" || $type == "read") && $mode == 0) {
                    $INFO['START'] = array();
                    $INFO['START'][0] = array();
                    $INFO['START'][0]['ACTION'] = $action;
                    $INFO['START'][0]['IMAGE']  = $INFO['ID'].'/icon.png';
                    $INFO['START'][0]['TITLE']  = $INFO['NAME'];
                }
            }

        } else {

            { /* create Interface Entry */
                $INTERFACE[$action]['TYPE'] = $type;
                $INTERFACE[$action]['MODE'] = 0;
                $INTERFACE[$action]['TEMPLATE'] = 'MESSAGE';
                if ($mode > 0) {
                    $INTERFACE[$action]['PERMISSION'] = $mode;
                } else {
                    $INTERFACE[$action]['PERMISSION'] = 0;
                }
                $INTERFACE[$action]['ONSUCCESS'] = array();
                $INTERFACE[$action]['ONERROR'] = array();
                $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $action;
                $INTERFACE[$action]['ONERROR']['GOTO'] = $action;
            }

            switch ($name)
            {
                case 'edit':
                    $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $this->_createActionName($id, 'read', $name, $DATABASE['TABLE']);
                    $INTERFACE[$action]['ONERROR']['GOTO'] = $this->_createActionName($id, 'read', $name, $DATABASE['TABLE']);
                break;
                case 'delete':
                    $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $this->_createActionName($id, 'read', 'edit', $DATABASE['TABLE']);
                    $INTERFACE[$action]['ONERROR']['GOTO'] = $this->_createActionName($id, 'read', 'edit', $DATABASE['TABLE']);
                break;
                case 'new':
                    $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $this->_createActionName($id, 'read', 'read', $DATABASE['TABLE']);
                    $INTERFACE[$action]['ONERROR']['GOTO'] = $this->_createActionName($id, 'default', $name, $DATABASE['TABLE']);
                break;
            }

        }

        if (!isset($AUTO_INTERFACE[$action])) {
            $AUTO_INTERFACE[$action] = $INTERFACE[$action];
        } else {
            $AUTO_INTERFACE[$action] = Hashtable::merge($AUTO_INTERFACE[$action], $INTERFACE[$action]);
        }
        $AUTO_INTERFACE[$action]['TABLE']  = $DATABASE['TABLE'];
        $AUTO_INTERFACE[$action]['PRESET'] = strtolower($name);
        return $action;
    }

    /**
     * @access private
     * @param scalar $id
     * @param scalar $type
     * @param scalar $name
     * @param scalar $table
     * @return string
     * @ignore
     */
    function _createActionName($id, $type, $name, $table)
    {
        return "${id}_${type}_${name}_${table}";
    }

    /**
     * Backup current settings
     *
     * @access private
     * @param array $ARGS
     * @return bool
     * @ignore
     */
    function _createBackup($ARGS)
    {
        global $YANA;
        $config = $YANA->plugins->get($this->name, 'CONFIGFILE');
        $config->read();
        if (!$config->exists()) {
            $config->create();
        };
        unset($ARGS[@session_name()]);
        unset($ARGS['id']);
        unset($ARGS['action']);
        return $config->insert($ARGS) && $config->write();
    }

}
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
