<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana Framework 3.0.2
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2015  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * «Singleton» user administration implementation
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 * @ignore
 */
class user_admin extends Singleton
{

    /**
     * @access  private
     * @var     string
     */
    var $user_name = "";

    /**
     * Constructor
     *
     * @access public
     */
    function __construct()
    {
        $this->checkSingletonConstraint();
        @session_start();
        global $YANA;
        if (isset($_SESSION['user_name'])) {
            $this->user_name = $_SESSION['user_name'];
        }
        $YANA->setVar("SESSION_ID", session_id());
        $YANA->setVar("SESSION_NAME", session_name());
    }

    /**
     * @access public
     * @return boolean
     * @ignore
     */
    function isEmpty()
    {
        global $YANA;
        return $YANA->session->isEmpty();
    }

    /**
     * @access  public
     * @param   string  $userId
     * @param   bool    $original
     * @return  bool
     * @ignore
     */
    function get_user($userId, $original = false)
    {
       settype($userId, "string");
       settype($original, "boolean");
       global $YANA;

       $USER = $YANA->session->getUser($userId);

       if (!$USER['USER_ID'] || !$USER['ACTIVE']) {
           $YANA->report( new ErrorMessage('user_unknown') );
           $YANA->exitTo('index');
           return false;
       } else {
           $YANA->setVar("USER", $USER);
           return true;
       }
    }

    /**
     * @access  public
     * @param   string  $userId
     * @param   bool    $use_thumb
     * @return  bool
     * @ignore
     */
    function get_profile_image($userId, $use_thumb = false)
    {
       settype($userId, "string");
       global $YANA;

       $USER = $YANA->session->getUser($userId);

       if (!$USER['USER_ID'] || !$USER['ACTIVE']) {
           $YANA->report( new ErrorMessage('user_unknown') );
           $YANA->exitTo('index');
           return false;
       } else {
           if (!empty($USER['USER_IMAGE']) && (strcasecmp($userId, $this->user_name) === 0 || $USER['USER_IMAGE_ACTIVE'])) {
               if (!$use_thumb) {
                   $image = new Image($USER['USER_IMAGE']);
               } else {
                   $image = new Image(str_replace('/image.', '/thumb.', $USER['USER_IMAGE']));
               }
           } else {
               $image = new Image($YANA->getVar('DATADIR').'userpic.gif');
           }
           $image->outputToScreen();
           exit;
       }
    }

    /**
     * @access  public
     * @param   string  $user_name
     * @param   array   $ROW
     * @return  bool
     * @ignore
     */
    function set_user_edit($user_name, $ROW)
    {
        settype($user_name, "string");
        settype($ROW, "array");
        global $YANA;

        if (strcasecmp($user_name, "ADMINISTRATOR") === 0) {
            if (!$ROW['active']) {
                $YANA->report( new Alert('USER_DELETE_ADMIN') );
                return false;
            } else if ($ROW['type'] !== 'ADMIN') {
                $YANA->report( new Alert('USER_DELETE_ADMIN') );
                return false;
            }
        }

        if (!$YANA->session->insertUser($user_name, $ROW)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * @access public
     * @param  array   $INPUT
     * @return boolean
     * @ignore
     */
    function set_profile_edit($INPUT)
    {
        settype($INPUT, "array");
        global $YANA;

        $INPUT = Hashtable::changeCase($INPUT,  CASE_UPPER);
        $FILES = array_change_key_case($_FILES, CASE_UPPER);

        /* untaint form input */
        foreach ($INPUT as $key => $element)
        {
            settype($element, "string");
            $element = strip_tags($element);
            $element = preg_replace("/\n/","[br]",$element);
            $element = preg_replace("/\s/"," ",$element);
            $INPUT[$key] = $element;
        } /* end foreach */
        unset($key);

        unset($INPUT["ACTIVE"]);
        unset($INPUT["USER_IMAGE_ACTIVE"]);
        unset($INPUT["PERM_GROUP"]);
        unset($INPUT["PERM_ID"]);
        unset($INPUT["USER_PWD"]);
        unset($INPUT["GROUP_ID"]);
        unset($INPUT["ACTION"]);
        unset($INPUT["ID"]);
        unset($INPUT[strtoupper(session_name())]);
        unset($INPUT["USER_ID"]);
        /* end untaint input */

        $INPUT["MODIFIED"] = time();

        /* handle images */
        $id = 'USER_IMAGE';

        unset($INPUT[$id]);
        if (!empty($FILES[$id]['name'])) {

            $INPUT[$id]                 = $id;
            $INPUT['USER_IMAGE_ACTIVE'] = false;

        } /* end image handling */

        /* Begin: create user data */

        $USER = $YANA->session->getUser($this->user_name);

        /**
         * add code to get rid of old images
         *
         * Note: this is currently not necessary as every
         * user has a unique image resource associated with him.
         */

        return $YANA->session->insertUser($this->user_name, $INPUT);
    }

    /**
     * @access  public
     * @param   string  $user_name
     * @return  bool
     * @ignore
     */
    function set_user_delete($user_name)
    {
       settype($user_name,"string");
       global $YANA;

       if (strtoupper($this->user_name) == strtoupper($user_name)) {
           $YANA->report( new Alert('USER_DELETE_SELF') );
           return false;
       } else if (strtoupper($user_name) == "ADMINISTRATOR") {
           $YANA->report( new Alert('USER_DELETE_ADMIN') );
           return false;
       } else if (!$YANA->session->removeUser($user_name)) {
           $YANA->report( new ErrorMessage('500') );
           return false;
       } else {
           return true;
       }
    }

    /**
     * @access  public
     * @param   string  $user_name
     * @return  bool
     * @ignore
     */
    function set_user_pwd($user_name)
    {
       settype($user_name,"string");
       global $YANA;
       $YANA->language->read("user");

       $pwt = $YANA->session->setPwd($user_name);

       if (!$pwt) {
           return false;
       } else {
           $user = $YANA->session->getUser($user_name);
           $YANA->setVar('PASSWORT', $pwt);
           $YANA->setVar('NAME', $user['USER_ID']);
           if (!empty($user['USER_MAIL'])) {
               user_admin::sendMail($user['USER_MAIL'], $YANA->skin->getFile("USER_PASSWORD_MAIL"));
           }
           return true;
       }
    }

    /**
     * @access public
     * @param  string   $user_name
     * @param  array    $INPUT
     * @return boolean
     * @ignore
     */
    function set_user_new()
    {
        global $YANA;

        $NEW_USER = FormCreator::getFormdata(FORM_NEW, new DbStructure('user'), "user");
        $user_name = $NEW_USER['user_id'];

        if (empty($user_name)) {
            $YANA->report( new Alert('INPUT_INVALID') );
            return false;
        } else if ($this->userExists($user_name)) {
            $YANA->report( new Alert('USER_ALREADY_EXISTS') );
            return false;
        } else if (!$YANA->session->insertUser($user_name, $NEW_USER)) {
            $YANA->report( new ErrorMessage('500') );
            return false;
        } else {
            return $this->set_user_pwd($user_name);
        }
    }

    /**
     * @access  public
     * @param   string   $user
     * @param   string   $mail
     * @return  bool
     * @ignore
     */
    function user_authentification($name, $mail)
    {
        settype($name, "string");
        settype($mail, "string");
        global $YANA;
 
        $newUser = $YANA->getDefault('USER');
        if (!is_array($newUser)) {
            $newUser = array();
        }

        if (empty($name)) {
            $YANA->report( new Alert('INPUT_INVALID') );
            $YANA->exitTo('');
            return false;
        } else if ($this->userExists($name)) {
            $YANA->report( new Alert('USER_ALREADY_EXISTS') );
            $YANA->exitTo('');
            return false;
        } else {

            $newUser['USER_MAIL'] = $mail;
            $newUser['USER_PWD'] = 'UNINITIALIZED';
            $newUser['GROUP_ID'] = 'REGISTERED';

            if (!$YANA->session->insertUser($name, $newUser)) {
                $YANA->report( new ErrorMessage('500') );
                $YANA->exitTo('');
                return false;
            } else {
                return $this->set_user_pwd($name);
            }
        }
    }

    /**
     * @access  public
     * @param   string  $user_name
     * @return  bool
     * @ignore
     */
    function userExists($user_name)
    {
        settype($user_name, "string");
        global $YANA;
        return $YANA->session->userExists($user_name);
    }

    /**
     * @access  public
     * @static
     * @param   string  $empfaenger
     * @param   string  $template
     * @ignore
     */
    function sendMail($empfaenger, $template)
    {
        settype($empfaenger, "string");
        settype($template,   "string");

        global $YANA;
        $heute = getdate();
        $mail = new Mailer($template);
        $mail->subject = $YANA->getVar("LANGUAGE.USER.MAIL_SUBJECT")."\n";
        $mail->insert('DATE', $heute['mday'] . '.' . $heute['mon'] . '.' . $heute['year']);
        $mail->send($empfaenger);
    }

}
// Copyright © 2015  Thomas Meyer  http://www.yanaframework.net
?>
